"""Delegated client for the networks domain."""

import json
from typing import Any, Literal, Optional, TypedDict, Union

from ..._internal import HttpClient
from ...base_auth_api import (
    BaseAuthApi,
    SignUserActionChallengeRequest,
    UserActionChallengeResponse,
)
from . import types as T


class DelegatedNetworksClient:
    """
    Delegated client for networks operations.

    This client separates user action signing into _init() and _complete() method pairs,
    allowing external systems to handle the signing process.
    """

    def __init__(self, http_client: HttpClient):
        self._http = http_client

    def estimate_fees(self, query: T.EstimateFeesQuery) -> TypedDict:
        """
        Estimate Fees.

        Gets real-time fee details for a given network, allowing users to make decisions based on their preferences for transaction speed/priority. Three levels of priority will be displayed: `slow`, `standard`, `fast`.

        Args:
        query: Query parameters.

        Returns:
            TypedDict: The API response.
        """
        return self._http.request(
            method="GET",
            path="/networks/fees",
            path_params={},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def read_contract(self, body: dict[str, Any]) -> T.ReadContractResponse:
        """
        Read Contract.

        Call a read-only function on a smart contract. In Solidity, this use the `view` keyword.

  <Note>
  Currently only works on EVM compatible chains.
  </Note>

        Args:
        body: Request body.

        Returns:
            T.ReadContractResponse: The API response.
        """
        return self._http.request(
            method="POST",
            path="/networks/read-contract",
            path_params={},
            query_params=None,
            body=body,
            requires_signature=False,
        )

    def get_canton_validator(self, network: Literal["canton", "canton-devnet", "canton-testnet"], validator_id: str) -> T.GetCantonValidatorResponse:
        """
        Get Canton Validator.

        Return a configured Canton Validator in your organization.

        Args:
        network: Path parameter.
        validator_id: Path parameter.

        Returns:
            T.GetCantonValidatorResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/networks/{network}/validators/{validatorId}",
            path_params={"network": network, "validatorId": validator_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def update_canton_validator_init(self, network: Literal["canton", "canton-devnet", "canton-testnet"], validator_id: str, body: T.UpdateCantonValidatorRequest) -> UserActionChallengeResponse:
        """
        Initialize Update Canton Validator.

        Creates a user action challenge for external signing.

        Args:
        network: Path parameter.
        validator_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/networks/{network}/validators/{validatorId}"
        path = path.replace("{network}", str(network))
        path = path.replace("{validatorId}", str(validator_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="PUT",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def update_canton_validator_complete(self, network: Literal["canton", "canton-devnet", "canton-testnet"], validator_id: str, body: T.UpdateCantonValidatorRequest, signed_challenge: SignUserActionChallengeRequest) -> T.UpdateCantonValidatorResponse:
        """
        Complete Update Canton Validator.

        Submits the signed challenge and makes the API request.

        Args:
        network: Path parameter.
        validator_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.UpdateCantonValidatorResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="PUT",
            path="/networks/{network}/validators/{validatorId}",
            path_params={"network": network, "validatorId": validator_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def delete_canton_validator_init(self, network: Literal["canton", "canton-devnet", "canton-testnet"], validator_id: str) -> UserActionChallengeResponse:
        """
        Initialize Delete Canton Validator.

        Creates a user action challenge for external signing.

        Args:
        network: Path parameter.
        validator_id: Path parameter.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/networks/{network}/validators/{validatorId}"
        path = path.replace("{network}", str(network))
        path = path.replace("{validatorId}", str(validator_id))
        payload = ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="DELETE",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def delete_canton_validator_complete(self, network: Literal["canton", "canton-devnet", "canton-testnet"], validator_id: str, signed_challenge: SignUserActionChallengeRequest) -> T.DeleteCantonValidatorResponse:
        """
        Complete Delete Canton Validator.

        Submits the signed challenge and makes the API request.

        Args:
        network: Path parameter.
        validator_id: Path parameter.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.DeleteCantonValidatorResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="DELETE",
            path="/networks/{network}/validators/{validatorId}",
            path_params={"network": network, "validatorId": validator_id},
            query_params=None,
            body=None,
            user_action=user_action_token,
        )

    def list_canton_validators(self, network: Literal["canton", "canton-devnet", "canton-testnet"], query: Optional[T.ListCantonValidatorsQuery] = None) -> T.ListCantonValidatorsResponse:
        """
        List Canton Validators.

        Retrieve the list of configured Canton Validators in your organization.

        Args:
        network: Path parameter.
        query: Query parameters.

        Returns:
            T.ListCantonValidatorsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/networks/{network}/validators",
            path_params={"network": network},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def create_canton_validator_init(self, network: Literal["canton", "canton-devnet", "canton-testnet"], body: dict[str, Any]) -> UserActionChallengeResponse:
        """
        Initialize Create Canton Validator.

        Creates a user action challenge for external signing.

        Args:
        network: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/networks/{network}/validators"
        path = path.replace("{network}", str(network))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def create_canton_validator_complete(self, network: Literal["canton", "canton-devnet", "canton-testnet"], body: dict[str, Any], signed_challenge: SignUserActionChallengeRequest) -> T.CreateCantonValidatorResponse:
        """
        Complete Create Canton Validator.

        Submits the signed challenge and makes the API request.

        Args:
        network: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.CreateCantonValidatorResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/networks/{network}/validators",
            path_params={"network": network},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )



"""Types for the permissions domain."""

from typing import Any, Literal, NotRequired, Optional, TypedDict, Union

class ListPermissionAssignmentsResponse(TypedDict, total=False):
    """listPermissionAssignments response."""

    items: list[TypedDict]
    next_page_token: NotRequired[str]

class AssignPermissionRequest(TypedDict, total=False):
    """assignPermission request body."""

    identity_id: str

class AssignPermissionResponse(TypedDict, total=False):
    """assignPermission response."""

    id: str
    permission_id: str
    identity_id: str
    is_immutable: bool
    date_created: str
    date_updated: str

class RevokePermissionQuery(TypedDict, total=False):
    """revokePermission query parameters."""

    force: NotRequired[bool]

class DeletePermissionRequest(TypedDict, total=False):
    """deletePermission request body."""

    is_archived: bool

class DeletePermissionResponse(TypedDict, total=False):
    """deletePermission response."""

    id: str
    name: str
    operations: list[str]
    status: Literal["Active"]
    is_immutable: bool
    is_archived: bool
    date_created: str
    date_updated: str

class ListPermissionsResponse(TypedDict, total=False):
    """listPermissions response."""

    items: list[TypedDict]
    next_page_token: NotRequired[str]

class ListPermissionsQuery(TypedDict, total=False):
    """listPermissions query parameters."""

    limit: NotRequired[str]
    pagination_token: NotRequired[str]

class CreatePermissionRequest(TypedDict, total=False):
    """createPermission request body."""

    name: str
    operations: list[Union[Literal["Registry:Addresses:Create", "Registry:Addresses:Delete", "Registry:Addresses:Read", "Registry:Addresses:Update", "Registry:ContractSchemas:Create", "Registry:ContractSchemas:Delete", "Registry:ContractSchemas:Read", "Auth:Logs:Read", "Auth:Users:Create", "Auth:Users:Read", "Auth:Users:Update", "Auth:Users:Activate", "Auth:Users:Deactivate", "Auth:Users:Delete", "Auth:ServiceAccounts:Create", "Auth:ServiceAccounts:Read", "Auth:ServiceAccounts:Update", "Auth:ServiceAccounts:Deactivate", "Auth:ServiceAccounts:Activate", "Auth:ServiceAccounts:Delete", "Auth:Pats:Create", "Auth:Register:Delegated", "Auth:Login:Delegated", "Auth:Recover:Delegated", "Agreements:Acceptance:Create", "Agreements:Acceptance:Read", "Events:Read", "Exchanges:Create", "Exchanges:Read", "Exchanges:Delete", "Exchanges:Deposits:Create", "Exchanges:Withdrawals:Create", "FeeSponsors:Create", "FeeSponsors:Read", "FeeSponsors:Update", "FeeSponsors:Delete", "FeeSponsors:Use", "Orgs:Read", "Orgs:Update", "Orgs:Settings:Read", "Orgs:Settings:Update", "Permissions:Archive", "Permissions:Create", "Permissions:Read", "Permissions:Update", "Permissions:Assign", "Permissions:Revoke", "Permissions:Assignments:Read", "Policies:Archive", "Policies:Create", "Policies:Read", "Policies:Update", "Policies:Approvals:Read", "Policies:Approvals:Approve", "Signers:ListSigners", "Stakes:Create", "Stakes:Read", "Stakes:Update", "Swaps:Create", "Swaps:Read", "Allocations:Create", "Allocations:Update", "Allocations:Read", "Keys:Create", "Keys:Delete", "Keys:Read", "Keys:Update", "Keys:Reuse", "Keys:Delegate", "Keys:Import", "Keys:Export", "Keys:Derive", "Keys:ChildKeys:Create", "Keys:Signatures:Create", "Keys:Signatures:Read", "KeyStores:Read", "Networks:CantonValidators:Create", "Networks:CantonValidators:Read", "Networks:CantonValidators:Update", "Networks:CantonValidators:Delete", "Wallets:Create", "Wallets:Read", "Wallets:Update", "Wallets:Tags:Add", "Wallets:Tags:Delete", "Wallets:Transactions:Create", "Wallets:Transactions:Read", "Wallets:Transfers:Create", "Wallets:Transfers:Read", "Wallets:Offers:Read", "Wallets:Offers:Settle", "Webhooks:Create", "Webhooks:Read", "Webhooks:Update", "Webhooks:Delete", "Webhooks:Ping", "Webhooks:Events:Read", "Billing:Read", "Billing:Write", "Analytics:Read"], Literal["Alias:Create", "Alias:Delete", "Alias:Read", "Alias:Update", "Wallets:GenerateSignature", "Wallets:BroadcastTransaction", "Auth:Action:Sign", "Auth:Apps:Read", "Auth:Apps:Create", "Auth:Apps:Update", "Auth:Creds:Create", "Auth:Creds:Read", "Auth:Creds:Update", "Auth:Creds:Code:Create", "Auth:Types:Application", "Auth:Types:Employee", "Auth:Types:EndUser", "Auth:Types:Pat", "Auth:Types:ServiceAccount", "Internal:Auth:Types:Staff", "Auth:Users:Delegate", "PermissionAssignments:Create", "PermissionAssignments:Read", "PermissionAssignments:Revoke"]]]

class CreatePermissionResponse(TypedDict, total=False):
    """createPermission response."""

    id: str
    name: str
    operations: list[str]
    status: Literal["Active"]
    is_immutable: bool
    is_archived: bool
    date_created: str
    date_updated: str

class GetPermissionResponse(TypedDict, total=False):
    """getPermission response."""

    id: str
    name: str
    operations: list[str]
    status: Literal["Active"]
    is_immutable: bool
    is_archived: bool
    date_created: str
    date_updated: str
    pending_change_request: NotRequired[TypedDict]

class UpdatePermissionRequest(TypedDict, total=False):
    """updatePermission request body."""

    name: NotRequired[str]
    operations: NotRequired[list[Union[Literal["Registry:Addresses:Create", "Registry:Addresses:Delete", "Registry:Addresses:Read", "Registry:Addresses:Update", "Registry:ContractSchemas:Create", "Registry:ContractSchemas:Delete", "Registry:ContractSchemas:Read", "Auth:Logs:Read", "Auth:Users:Create", "Auth:Users:Read", "Auth:Users:Update", "Auth:Users:Activate", "Auth:Users:Deactivate", "Auth:Users:Delete", "Auth:ServiceAccounts:Create", "Auth:ServiceAccounts:Read", "Auth:ServiceAccounts:Update", "Auth:ServiceAccounts:Deactivate", "Auth:ServiceAccounts:Activate", "Auth:ServiceAccounts:Delete", "Auth:Pats:Create", "Auth:Register:Delegated", "Auth:Login:Delegated", "Auth:Recover:Delegated", "Agreements:Acceptance:Create", "Agreements:Acceptance:Read", "Events:Read", "Exchanges:Create", "Exchanges:Read", "Exchanges:Delete", "Exchanges:Deposits:Create", "Exchanges:Withdrawals:Create", "FeeSponsors:Create", "FeeSponsors:Read", "FeeSponsors:Update", "FeeSponsors:Delete", "FeeSponsors:Use", "Orgs:Read", "Orgs:Update", "Orgs:Settings:Read", "Orgs:Settings:Update", "Permissions:Archive", "Permissions:Create", "Permissions:Read", "Permissions:Update", "Permissions:Assign", "Permissions:Revoke", "Permissions:Assignments:Read", "Policies:Archive", "Policies:Create", "Policies:Read", "Policies:Update", "Policies:Approvals:Read", "Policies:Approvals:Approve", "Signers:ListSigners", "Stakes:Create", "Stakes:Read", "Stakes:Update", "Swaps:Create", "Swaps:Read", "Allocations:Create", "Allocations:Update", "Allocations:Read", "Keys:Create", "Keys:Delete", "Keys:Read", "Keys:Update", "Keys:Reuse", "Keys:Delegate", "Keys:Import", "Keys:Export", "Keys:Derive", "Keys:ChildKeys:Create", "Keys:Signatures:Create", "Keys:Signatures:Read", "KeyStores:Read", "Networks:CantonValidators:Create", "Networks:CantonValidators:Read", "Networks:CantonValidators:Update", "Networks:CantonValidators:Delete", "Wallets:Create", "Wallets:Read", "Wallets:Update", "Wallets:Tags:Add", "Wallets:Tags:Delete", "Wallets:Transactions:Create", "Wallets:Transactions:Read", "Wallets:Transfers:Create", "Wallets:Transfers:Read", "Wallets:Offers:Read", "Wallets:Offers:Settle", "Webhooks:Create", "Webhooks:Read", "Webhooks:Update", "Webhooks:Delete", "Webhooks:Ping", "Webhooks:Events:Read", "Billing:Read", "Billing:Write", "Analytics:Read"], Literal["Alias:Create", "Alias:Delete", "Alias:Read", "Alias:Update", "Wallets:GenerateSignature", "Wallets:BroadcastTransaction", "Auth:Action:Sign", "Auth:Apps:Read", "Auth:Apps:Create", "Auth:Apps:Update", "Auth:Creds:Create", "Auth:Creds:Read", "Auth:Creds:Update", "Auth:Creds:Code:Create", "Auth:Types:Application", "Auth:Types:Employee", "Auth:Types:EndUser", "Auth:Types:Pat", "Auth:Types:ServiceAccount", "Internal:Auth:Types:Staff", "Auth:Users:Delegate", "PermissionAssignments:Create", "PermissionAssignments:Read", "PermissionAssignments:Revoke"]]]]

class UpdatePermissionResponse(TypedDict, total=False):
    """updatePermission response."""

    id: str
    name: str
    operations: list[str]
    status: Literal["Active"]
    is_immutable: bool
    is_archived: bool
    date_created: str
    date_updated: str



"""Client for the allocations domain."""

from typing import Any, Literal, Optional, TypedDict, Union

from ..._internal import HttpClient
from . import types as T


class AllocationsClient:
    """Client for allocations operations."""

    def __init__(self, http_client: HttpClient):
        self._http = http_client

    def list_allocations(self, query: Optional[T.ListAllocationsQuery] = None) -> T.ListAllocationsResponse:
        """
        List Allocations.

        Args:
        query: Query parameters.

        Returns:
            T.ListAllocationsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/allocations",
            path_params={},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def create_allocation(self, body: dict[str, Any]) -> T.CreateAllocationResponse:
        """
        Create Allocation.

        Create a new allocation.

        Args:
        body: Request body.

        Returns:
            T.CreateAllocationResponse: The API response.
        """
        return self._http.request(
            method="POST",
            path="/allocations",
            path_params={},
            query_params=None,
            body=body,
            requires_signature=True,
        )

    def list_allocation_actions(self, allocation_id: str, query: Optional[T.ListAllocationActionsQuery] = None) -> T.ListAllocationActionsResponse:
        """
        List Allocation Actions.

        Retrieve the list of actions for a specific allocation.

        Args:
        allocation_id: Unique identifier for the allocation investment.
        query: Query parameters.

        Returns:
            T.ListAllocationActionsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/allocations/{allocationId}/actions",
            path_params={"allocationId": allocation_id},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def create_allocation_action(self, allocation_id: str, body: T.CreateAllocationActionRequest) -> T.CreateAllocationActionResponse:
        """
        Create Allocation Action.

        Create a new action for an existing allocation.

        Args:
        allocation_id: Unique identifier for the allocation investment.
        body: Request body.

        Returns:
            T.CreateAllocationActionResponse: The API response.
        """
        return self._http.request(
            method="POST",
            path="/allocations/{allocationId}/actions",
            path_params={"allocationId": allocation_id},
            query_params=None,
            body=body,
            requires_signature=True,
        )

    def get_allocation(self, allocation_id: str) -> T.GetAllocationResponse:
        """
        Get Allocation.

        Retrieve the details of a specific allocation.

        Args:
        allocation_id: Unique identifier for the allocation investment.

        Returns:
            T.GetAllocationResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/allocations/{allocationId}",
            path_params={"allocationId": allocation_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )



"""Delegated client for the wallets domain."""

import json
from typing import Any, Literal, Optional, TypedDict, Union
from warnings import deprecated


from ..._internal import HttpClient
from ...base_auth_api import (
    BaseAuthApi,
    SignUserActionChallengeRequest,
    UserActionChallengeResponse,
)
from . import types as T


class DelegatedWalletsClient:
    """
    Delegated client for wallets operations.

    This client separates user action signing into _init() and _complete() method pairs,
    allowing external systems to handle the signing process.
    """

    def __init__(self, http_client: HttpClient):
        self._http = http_client

    def activate_wallet_init(self, wallet_id: str, body: dict[str, Any]) -> UserActionChallengeResponse:
        """
        Initialize Activate Wallet.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Wallet id.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/activate"
        path = path.replace("{walletId}", str(wallet_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def activate_wallet_complete(self, wallet_id: str, body: dict[str, Any], signed_challenge: SignUserActionChallengeRequest) -> T.ActivateWalletResponse:
        """
        Complete Activate Wallet.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Wallet id.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.ActivateWalletResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/wallets/{walletId}/activate",
            path_params={"walletId": wallet_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def list_transactions(self, wallet_id: str, query: Optional[T.ListTransactionsQuery] = None) -> T.ListTransactionsResponse:
        """
        List Transactions.

        Retrieves a list of transactions requests for the specified wallet.

        Args:
        wallet_id: Path parameter.
        query: Query parameters.

        Returns:
            T.ListTransactionsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets/{walletId}/transactions",
            path_params={"walletId": wallet_id},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def sign_and_broadcast_transaction_init(self, wallet_id: str, body: dict[str, Any]) -> UserActionChallengeResponse:
        """
        Initialize Sign and Broadcast Transaction.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/transactions"
        path = path.replace("{walletId}", str(wallet_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def sign_and_broadcast_transaction_complete(self, wallet_id: str, body: dict[str, Any], signed_challenge: SignUserActionChallengeRequest) -> T.SignAndBroadcastTransactionResponse:
        """
        Complete Sign and Broadcast Transaction.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.SignAndBroadcastTransactionResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/wallets/{walletId}/transactions",
            path_params={"walletId": wallet_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def cancel_transaction_init(self, wallet_id: str, transaction_id: str) -> UserActionChallengeResponse:
        """
        Initialize Cancel Transaction.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Wallet id.
        transaction_id: Transaction id.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/transactions/{transactionId}/cancel"
        path = path.replace("{walletId}", str(wallet_id))
        path = path.replace("{transactionId}", str(transaction_id))
        payload = ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def cancel_transaction_complete(self, wallet_id: str, transaction_id: str, signed_challenge: SignUserActionChallengeRequest) -> T.CancelTransactionResponse:
        """
        Complete Cancel Transaction.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Wallet id.
        transaction_id: Transaction id.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.CancelTransactionResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/wallets/{walletId}/transactions/{transactionId}/cancel",
            path_params={"walletId": wallet_id, "transactionId": transaction_id},
            query_params=None,
            body=None,
            user_action=user_action_token,
        )

    def cancel_transfer_init(self, wallet_id: str, transfer_id: str) -> UserActionChallengeResponse:
        """
        Initialize Cancel Transfer.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Wallet id.
        transfer_id: Transfer id.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/transfers/{transferId}/cancel"
        path = path.replace("{walletId}", str(wallet_id))
        path = path.replace("{transferId}", str(transfer_id))
        payload = ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def cancel_transfer_complete(self, wallet_id: str, transfer_id: str, signed_challenge: SignUserActionChallengeRequest) -> T.CancelTransferResponse:
        """
        Complete Cancel Transfer.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Wallet id.
        transfer_id: Transfer id.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.CancelTransferResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/wallets/{walletId}/transfers/{transferId}/cancel",
            path_params={"walletId": wallet_id, "transferId": transfer_id},
            query_params=None,
            body=None,
            user_action=user_action_token,
        )

    def speed_up_transaction_init(self, wallet_id: str, transaction_id: str) -> UserActionChallengeResponse:
        """
        Initialize Speed Up Transaction.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Wallet id.
        transaction_id: Transaction id.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/transactions/{transactionId}/speed-up"
        path = path.replace("{walletId}", str(wallet_id))
        path = path.replace("{transactionId}", str(transaction_id))
        payload = ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def speed_up_transaction_complete(self, wallet_id: str, transaction_id: str, signed_challenge: SignUserActionChallengeRequest) -> T.SpeedUpTransactionResponse:
        """
        Complete Speed Up Transaction.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Wallet id.
        transaction_id: Transaction id.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.SpeedUpTransactionResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/wallets/{walletId}/transactions/{transactionId}/speed-up",
            path_params={"walletId": wallet_id, "transactionId": transaction_id},
            query_params=None,
            body=None,
            user_action=user_action_token,
        )

    def speed_up_transfer_init(self, wallet_id: str, transfer_id: str) -> UserActionChallengeResponse:
        """
        Initialize Speed Up Transfer.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Wallet id.
        transfer_id: Transfer id.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/transfers/{transferId}/speed-up"
        path = path.replace("{walletId}", str(wallet_id))
        path = path.replace("{transferId}", str(transfer_id))
        payload = ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def speed_up_transfer_complete(self, wallet_id: str, transfer_id: str, signed_challenge: SignUserActionChallengeRequest) -> T.SpeedUpTransferResponse:
        """
        Complete Speed Up Transfer.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Wallet id.
        transfer_id: Transfer id.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.SpeedUpTransferResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/wallets/{walletId}/transfers/{transferId}/speed-up",
            path_params={"walletId": wallet_id, "transferId": transfer_id},
            query_params=None,
            body=None,
            user_action=user_action_token,
        )

    def list_wallets(self, query: Optional[T.ListWalletsQuery] = None) -> T.ListWalletsResponse:
        """
        List Wallets.

        Retrieves the list of Wallets in your organization. You can filter the results by owner (either by owner id or owner username). Pagination is supported via limit and paginationToken parameters.

        Args:
        query: Query parameters.

        Returns:
            T.ListWalletsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets",
            path_params={},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def create_wallet_init(self, body: T.CreateWalletRequest) -> UserActionChallengeResponse:
        """
        Initialize Create Wallet.

        Creates a user action challenge for external signing.

        Args:
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets"
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def create_wallet_complete(self, body: T.CreateWalletRequest, signed_challenge: SignUserActionChallengeRequest) -> T.CreateWalletResponse:
        """
        Complete Create Wallet.

        Submits the signed challenge and makes the API request.

        Args:
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.CreateWalletResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/wallets",
            path_params={},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def delegate_wallet_init(self, wallet_id: str, body: T.DelegateWalletRequest) -> UserActionChallengeResponse:
        """
        Initialize Delegate Wallet.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/delegate"
        path = path.replace("{walletId}", str(wallet_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def delegate_wallet_complete(self, wallet_id: str, body: T.DelegateWalletRequest, signed_challenge: SignUserActionChallengeRequest) -> T.DelegateWalletResponse:
        """
        Complete Delegate Wallet.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.DelegateWalletResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/wallets/{walletId}/delegate",
            path_params={"walletId": wallet_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def export_wallet_init(self, wallet_id: str, body: T.ExportWalletRequest) -> UserActionChallengeResponse:
        """
        Initialize Export Wallet.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/export"
        path = path.replace("{walletId}", str(wallet_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def export_wallet_complete(self, wallet_id: str, body: T.ExportWalletRequest, signed_challenge: SignUserActionChallengeRequest) -> T.ExportWalletResponse:
        """
        Complete Export Wallet.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.ExportWalletResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/wallets/{walletId}/export",
            path_params={"walletId": wallet_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    @deprecated("This endpoint is deprecated.")
    def list_signatures(self, wallet_id: str, query: Optional[T.ListSignaturesQuery] = None) -> T.ListSignaturesResponse:
        """
        List Signatures.

        List all signature requests for a specific wallet.
  
<Danger>
Generating Signatures from a Wallet is deprecated. Please use the [Keys](https://docs.dfns.co/api-reference/keys) endpoints instead.
</Danger>

        Args:
        wallet_id: Path parameter.
        query: Query parameters.

        Returns:
            T.ListSignaturesResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets/{walletId}/signatures",
            path_params={"walletId": wallet_id},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def generate_signature_init(self, wallet_id: str, body: dict[str, Any]) -> UserActionChallengeResponse:
        """
        Initialize Generate Signature.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/signatures"
        path = path.replace("{walletId}", str(wallet_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def generate_signature_complete(self, wallet_id: str, body: dict[str, Any], signed_challenge: SignUserActionChallengeRequest) -> T.GenerateSignatureResponse:
        """
        Complete Generate Signature.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.GenerateSignatureResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/wallets/{walletId}/signatures",
            path_params={"walletId": wallet_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    @deprecated("This endpoint is deprecated.")
    def get_signature(self, wallet_id: str, signature_id: str) -> T.GetSignatureResponse:
        """
        Get Signature.

        Retrieves a Transaction Request information by its ID.

<Danger>
Generating Signatures from a Wallet is deprecated. Please use the [Keys](https://docs.dfns.co/api-reference/keys) endpoints instead.
</Danger>

        Args:
        wallet_id: Path parameter.
        signature_id: Path parameter.

        Returns:
            T.GetSignatureResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets/{walletId}/signatures/{signatureId}",
            path_params={"walletId": wallet_id, "signatureId": signature_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def get_transaction(self, wallet_id: str, transaction_id: str) -> T.GetTransactionResponse:
        """
        Get Transaction.

        Retrieve information about a specific transaction.

        Args:
        wallet_id: Path parameter.
        transaction_id: Path parameter.

        Returns:
            T.GetTransactionResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets/{walletId}/transactions/{transactionId}",
            path_params={"walletId": wallet_id, "transactionId": transaction_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def get_transfer(self, wallet_id: str, transfer_id: str) -> T.GetTransferResponse:
        """
        Get Transfer.

        Retrieves a Wallet Transfer Request by its ID.

        Args:
        wallet_id: Path parameter.
        transfer_id: Path parameter.

        Returns:
            T.GetTransferResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets/{walletId}/transfers/{transferId}",
            path_params={"walletId": wallet_id, "transferId": transfer_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def get_wallet(self, wallet_id: str) -> T.GetWalletResponse:
        """
        Get Wallet.

        Retrieves a Wallet information by its ID.

        Args:
        wallet_id: Path parameter.

        Returns:
            T.GetWalletResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets/{walletId}",
            path_params={"walletId": wallet_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def update_wallet_init(self, wallet_id: str, body: T.UpdateWalletRequest) -> UserActionChallengeResponse:
        """
        Initialize Update Wallet.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}"
        path = path.replace("{walletId}", str(wallet_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="PUT",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def update_wallet_complete(self, wallet_id: str, body: T.UpdateWalletRequest, signed_challenge: SignUserActionChallengeRequest) -> T.UpdateWalletResponse:
        """
        Complete Update Wallet.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.UpdateWalletResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="PUT",
            path="/wallets/{walletId}",
            path_params={"walletId": wallet_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def get_wallet_assets(self, wallet_id: str, query: Optional[T.GetWalletAssetsQuery] = None) -> T.GetWalletAssetsResponse:
        """
        Get Wallet Assets.

        Retrieves a list of assets owned by the specified wallet.  Return values vary by chain as shown below.

        Args:
        wallet_id: Path parameter.
        query: Query parameters.

        Returns:
            T.GetWalletAssetsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets/{walletId}/assets",
            path_params={"walletId": wallet_id},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def get_wallet_history(self, wallet_id: str, query: Optional[T.GetWalletHistoryQuery] = None) -> T.GetWalletHistoryResponse:
        """
        Get Wallet History.

        Retrieves a list of historical on chain activities for the specified wallet.
  
The list reflects the indexed on-chain activity: it includes confirmed transactions only. 

If you need to list your on-going or failed transactions please use the related endpoints (
[List Transfers](https://docs.dfns.co/api-reference/wallets/list-transfers) or 
[List Transactions](https://docs.dfns.co/api-reference/wallets/list-transactions)
depending on the API you are using).

        Args:
        wallet_id: Wallet you want to get the history from.
        query: Query parameters.

        Returns:
            T.GetWalletHistoryResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets/{walletId}/history",
            path_params={"walletId": wallet_id},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def get_wallet_nfts(self, wallet_id: str) -> T.GetWalletNftsResponse:
        """
        Get Wallet Nfts.

        Retrieves a list of NFTs owned by the specified Wallet.

        Args:
        wallet_id: Path parameter.

        Returns:
            T.GetWalletNftsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets/{walletId}/nfts",
            path_params={"walletId": wallet_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def import_wallet_init(self, body: T.ImportWalletRequest) -> UserActionChallengeResponse:
        """
        Initialize Import Wallet.

        Creates a user action challenge for external signing.

        Args:
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/import"
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def import_wallet_complete(self, body: T.ImportWalletRequest, signed_challenge: SignUserActionChallengeRequest) -> T.ImportWalletResponse:
        """
        Complete Import Wallet.

        Submits the signed challenge and makes the API request.

        Args:
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.ImportWalletResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/wallets/import",
            path_params={},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def list_transfers(self, wallet_id: str, query: Optional[T.ListTransfersQuery] = None) -> T.ListTransfersResponse:
        """
        List Transfers.

        Retrieves a list of transfer requests for the specified wallet.

        Args:
        wallet_id: Path parameter.
        query: Query parameters.

        Returns:
            T.ListTransfersResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets/{walletId}/transfers",
            path_params={"walletId": wallet_id},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def transfer_asset_init(self, wallet_id: str, body: dict[str, Any]) -> UserActionChallengeResponse:
        """
        Initialize Transfer Asset.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: The source wallet id (`wa-...`).
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/transfers"
        path = path.replace("{walletId}", str(wallet_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def transfer_asset_complete(self, wallet_id: str, body: dict[str, Any], signed_challenge: SignUserActionChallengeRequest) -> T.TransferAssetResponse:
        """
        Complete Transfer Asset.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: The source wallet id (`wa-...`).
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.TransferAssetResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/wallets/{walletId}/transfers",
            path_params={"walletId": wallet_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def tag_wallet_init(self, wallet_id: str, body: T.TagWalletRequest) -> UserActionChallengeResponse:
        """
        Initialize Tag Wallet.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/tags"
        path = path.replace("{walletId}", str(wallet_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="PUT",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def tag_wallet_complete(self, wallet_id: str, body: T.TagWalletRequest, signed_challenge: SignUserActionChallengeRequest) -> T.TagWalletResponse:
        """
        Complete Tag Wallet.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.TagWalletResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="PUT",
            path="/wallets/{walletId}/tags",
            path_params={"walletId": wallet_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def untag_wallet_init(self, wallet_id: str, body: T.UntagWalletRequest) -> UserActionChallengeResponse:
        """
        Initialize Untag Wallet.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/tags"
        path = path.replace("{walletId}", str(wallet_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="DELETE",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def untag_wallet_complete(self, wallet_id: str, body: T.UntagWalletRequest, signed_challenge: SignUserActionChallengeRequest) -> T.UntagWalletResponse:
        """
        Complete Untag Wallet.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.UntagWalletResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="DELETE",
            path="/wallets/{walletId}/tags",
            path_params={"walletId": wallet_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def get_offer(self, wallet_id: str, offer_id: str) -> T.GetOfferResponse:
        """
        Get Offer.

        Retrieve information about a specific offer received on your wallet.

        Args:
        wallet_id: Wallet id.
        offer_id: Offer id.

        Returns:
            T.GetOfferResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets/{walletId}/offers/{offerId}",
            path_params={"walletId": wallet_id, "offerId": offer_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def list_offers(self, wallet_id: str, query: Optional[T.ListOffersQuery] = None) -> T.ListOffersResponse:
        """
        List Offers.

        List all offers received on a specific wallet.

        Args:
        wallet_id: Wallet id.
        query: Query parameters.

        Returns:
            T.ListOffersResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets/{walletId}/offers",
            path_params={"walletId": wallet_id},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def accept_offer_init(self, wallet_id: str, offer_id: str) -> UserActionChallengeResponse:
        """
        Initialize Accept Offer.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Wallet id.
        offer_id: Offer id.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/offers/{offerId}/accept"
        path = path.replace("{walletId}", str(wallet_id))
        path = path.replace("{offerId}", str(offer_id))
        payload = ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="PUT",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def accept_offer_complete(self, wallet_id: str, offer_id: str, signed_challenge: SignUserActionChallengeRequest) -> T.AcceptOfferResponse:
        """
        Complete Accept Offer.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Wallet id.
        offer_id: Offer id.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.AcceptOfferResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="PUT",
            path="/wallets/{walletId}/offers/{offerId}/accept",
            path_params={"walletId": wallet_id, "offerId": offer_id},
            query_params=None,
            body=None,
            user_action=user_action_token,
        )

    def reject_offer_init(self, wallet_id: str, offer_id: str) -> UserActionChallengeResponse:
        """
        Initialize Reject Offer.

        Creates a user action challenge for external signing.

        Args:
        wallet_id: Wallet id.
        offer_id: Offer id.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/wallets/{walletId}/offers/{offerId}/reject"
        path = path.replace("{walletId}", str(wallet_id))
        path = path.replace("{offerId}", str(offer_id))
        payload = ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="PUT",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def reject_offer_complete(self, wallet_id: str, offer_id: str, signed_challenge: SignUserActionChallengeRequest) -> T.RejectOfferResponse:
        """
        Complete Reject Offer.

        Submits the signed challenge and makes the API request.

        Args:
        wallet_id: Wallet id.
        offer_id: Offer id.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.RejectOfferResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="PUT",
            path="/wallets/{walletId}/offers/{offerId}/reject",
            path_params={"walletId": wallet_id, "offerId": offer_id},
            query_params=None,
            body=None,
            user_action=user_action_token,
        )

    def list_org_wallet_history(self, query: T.ListOrgWalletHistoryQuery) -> Union[TypedDict, str]:
        """
        List Org Wallet History.

        Retrieve the transaction history across all wallets within a specified timeframe.

        Args:
        query: Query parameters.

        Returns:
            Union[TypedDict, str]: The API response.
        """
        return self._http.request(
            method="GET",
            path="/wallets/all/history",
            path_params={},
            query_params=query,
            body=None,
            requires_signature=False,
        )



"""Client for the exchanges domain."""

from typing import Any, Literal, Optional, TypedDict, Union

from ..._internal import HttpClient
from . import types as T


class ExchangesClient:
    """Client for exchanges operations."""

    def __init__(self, http_client: HttpClient):
        self._http = http_client

    def get_exchange(self, exchange_id: str) -> T.GetExchangeResponse:
        """
        Get Exchange.

        Retrieve the details of a specific exchange integration configuration.

        Args:
        exchange_id: Path parameter.

        Returns:
            T.GetExchangeResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/exchanges/{exchangeId}",
            path_params={"exchangeId": exchange_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def delete_exchange(self, exchange_id: str) -> T.DeleteExchangeResponse:
        """
        Delete Exchange.

        Delete the exchange configuration from your organization.

        Args:
        exchange_id: Path parameter.

        Returns:
            T.DeleteExchangeResponse: The API response.
        """
        return self._http.request(
            method="DELETE",
            path="/exchanges/{exchangeId}",
            path_params={"exchangeId": exchange_id},
            query_params=None,
            body=None,
            requires_signature=True,
        )

    def list_exchanges(self, query: Optional[T.ListExchangesQuery] = None) -> T.ListExchangesResponse:
        """
        List Exchanges.

        List all configured exchange integrations.

        Args:
        query: Query parameters.

        Returns:
            T.ListExchangesResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/exchanges",
            path_params={},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def create_exchange(self, body: T.CreateExchangeRequest) -> T.CreateExchangeResponse:
        """
        Create Exchange.

        Link your organization with a cryptocurrency exchange.

        Args:
        body: Request body.

        Returns:
            T.CreateExchangeResponse: The API response.
        """
        return self._http.request(
            method="POST",
            path="/exchanges",
            path_params={},
            query_params=None,
            body=body,
            requires_signature=True,
        )

    def list_accounts(self, exchange_id: str, query: Optional[T.ListAccountsQuery] = None) -> T.ListAccountsResponse:
        """
        List Accounts.

        Get a list of accounts for a specific exchange.

        Args:
        exchange_id: Path parameter.
        query: Query parameters.

        Returns:
            T.ListAccountsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/exchanges/{exchangeId}/accounts",
            path_params={"exchangeId": exchange_id},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def list_account_assets(self, exchange_id: str, account_id: str, query: Optional[T.ListAccountAssetsQuery] = None) -> T.ListAccountAssetsResponse:
        """
        List Account Assets.

        Retrieve the list of assets for a specific account on a specific exchange.

        Args:
        exchange_id: Path parameter.
        account_id: Path parameter.
        query: Query parameters.

        Returns:
            T.ListAccountAssetsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/exchanges/{exchangeId}/accounts/{accountId}/assets",
            path_params={"exchangeId": exchange_id, "accountId": account_id},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def list_asset_withdrawal_networks(self, exchange_id: str, account_id: str, asset: str) -> list[TypedDict]:
        """
        List Asset Withdrawal Networks.

        Args:
        exchange_id: Path parameter.
        account_id: Path parameter.
        asset: Path parameter.

        Returns:
            list[TypedDict]: The API response.
        """
        return self._http.request(
            method="GET",
            path="/exchanges/{exchangeId}/accounts/{accountId}/assets/{asset}/withdrawal-networks",
            path_params={"exchangeId": exchange_id, "accountId": account_id, "asset": asset},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def create_exchange_deposit(self, exchange_id: str, account_id: str, body: dict[str, Any]) -> T.CreateExchangeDepositResponse:
        """
        Create Exchange Deposit.

        Creates a new exchange deposit transaction.

        Args:
        exchange_id: The exchange id obtained from the Create Exchange endpoint. Ex: `ex-1f04s-lqc9q-xxxxxxxxxxxxxxxx`
        account_id: Unique identifier for the account like "spot"
        body: Request body.

        Returns:
            T.CreateExchangeDepositResponse: The API response.
        """
        return self._http.request(
            method="POST",
            path="/exchanges/{exchangeId}/accounts/{accountId}/deposits",
            path_params={"exchangeId": exchange_id, "accountId": account_id},
            query_params=None,
            body=body,
            requires_signature=True,
        )

    def create_exchange_withdrawal(self, exchange_id: str, account_id: str, body: dict[str, Any]) -> T.CreateExchangeWithdrawalResponse:
        """
        Create Exchange Withdrawal.

        Creates a new exchange withdrawal transaction.

        Args:
        exchange_id: The exchange id obtained from the Create Exchange endpoint. Ex: `ex-1f04s-lqc9q-xxxxxxxxxxxxxxxx`
        account_id: Unique identifier for the account like "spot"
        body: Request body.

        Returns:
            T.CreateExchangeWithdrawalResponse: The API response.
        """
        return self._http.request(
            method="POST",
            path="/exchanges/{exchangeId}/accounts/{accountId}/withdrawals",
            path_params={"exchangeId": exchange_id, "accountId": account_id},
            query_params=None,
            body=body,
            requires_signature=True,
        )



"""HTTP client for making API requests."""

import json
from typing import Any, Optional, TypeVar, TYPE_CHECKING
from urllib.parse import urlencode

import httpx

from dfns_sdk.types import DfnsClientConfig, DfnsDelegatedClientConfig, DfnsError

if TYPE_CHECKING:
    from dfns_sdk.auth import Signer

T = TypeVar("T")



class HttpClient:
    """HTTP client for Dfns API requests."""

    def __init__(self, config: "DfnsClientConfig | DfnsDelegatedClientConfig"):
        self.config = config
        self._client = httpx.Client(
            base_url=config.base_url,
            timeout=30.0,
        )

    def _build_headers(self, user_action_token: Optional[str] = None) -> dict[str, str]:
        """Build request headers."""
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.config.auth_token}",
            **self.config.headers,
        }

        if user_action_token:
            headers["X-DFNS-USERACTION"] = user_action_token

        return headers

    def _build_url(
        self,
        path: str,
        path_params: Optional[dict[str, Any]] = None,
        query_params: Optional[dict[str, Any]] = None,
    ) -> str:
        """Build the full URL with path and query parameters."""
        url = path

        if path_params:
            for key, value in path_params.items():
                url = url.replace(f"{{{key}}}", str(value))

        if query_params:
            filtered_params = {k: v for k, v in query_params.items() if v is not None}
            if filtered_params:
                url = f"{url}?{urlencode(filtered_params)}"

        return url

    def _handle_response(self, response: httpx.Response) -> Any:
        """Handle API response and raise errors if needed."""
        if response.status_code >= 400:
            try:
                error_data = response.json()
                raise DfnsError(
                    message=error_data.get("message", "Unknown error"),
                    status_code=response.status_code,
                    error_code=error_data.get("error"),
                    details=error_data.get("details"),
                )
            except json.JSONDecodeError:
                raise DfnsError(
                    message=response.text or "Unknown error",
                    status_code=response.status_code,
                )

        if response.status_code == 204 or not response.content:
            return None

        return response.json()

    def _get_user_action_token(
        self,
        method: str,
        path: str,
        body: Optional[Any] = None,
    ) -> str:
        """
        Get a user action token by creating and signing a challenge.

        Args:
            method: HTTP method.
            path: Request path.
            body: Request body.

        Returns:
            The user action token to include in the request header.

        Raises:
            DfnsError: If no signer is configured or signing fails.
        """
        if not self.config.signer:
            raise DfnsError(
                message="Signer required for this operation. Configure a signer in DfnsClientConfig.",
                status_code=None,
                error_code="SIGNER_REQUIRED",
            )

        # Step 1: Create user action challenge
        challenge_body = {
            "userActionPayload": json.dumps(body, separators=(",", ":")) if body else "",
            "userActionHttpMethod": method,
            "userActionHttpPath": path,
            "userActionServerKind": "Api",
        }

        challenge_response = self._client.request(
            method="POST",
            url="/auth/action/init",
            headers=self._build_headers(),
            json=challenge_body,
        )
        challenge = self._handle_response(challenge_response)

        # Step 2: Sign the challenge
        assertion = self.config.signer.sign(challenge)

        # Step 3: Submit signed challenge to get user action token
        signature_body = {
            "challengeIdentifier": challenge["challengeIdentifier"],
            "firstFactor": assertion,
        }

        signature_response = self._client.request(
            method="POST",
            url="/auth/action",
            headers=self._build_headers(),
            json=signature_body,
        )
        result = self._handle_response(signature_response)

        return result["userAction"]

    def request(
        self,
        method: str,
        path: str,
        path_params: Optional[dict[str, Any]] = None,
        query_params: Optional[dict[str, Any]] = None,
        body: Optional[Any] = None,
        requires_signature: bool = False,
    ) -> Any:
        """Make an HTTP request to the API."""
        url = self._build_url(path, path_params, query_params)

        # Get user action token if required
        user_action_token = None
        if requires_signature:
            # Use the path with params substituted for signing
            signing_path = path
            if path_params:
                for key, value in path_params.items():
                    signing_path = signing_path.replace(f"{{{key}}}", str(value))
            user_action_token = self._get_user_action_token(method, signing_path, body)

        headers = self._build_headers(user_action_token)

        response = self._client.request(
            method=method,
            url=url,
            headers=headers,
            json=body if body is not None else None,
        )

        return self._handle_response(response)

    def request_with_user_action(
        self,
        method: str,
        path: str,
        path_params: Optional[dict[str, Any]] = None,
        query_params: Optional[dict[str, Any]] = None,
        body: Optional[Any] = None,
        user_action: str = "",
    ) -> Any:
        """
        Make an HTTP request with a pre-signed user action token.

        This method is used by delegated clients where signing is handled externally.

        Args:
            method: HTTP method.
            path: Request path.
            path_params: Path parameters to substitute.
            query_params: Query parameters.
            body: Request body.
            user_action: Pre-signed user action token.

        Returns:
            The API response.
        """
        url = self._build_url(path, path_params, query_params)
        headers = self._build_headers(user_action if user_action else None)

        response = self._client.request(
            method=method,
            url=url,
            headers=headers,
            json=body if body is not None else None,
        )

        return self._handle_response(response)

    def close(self) -> None:
        """Close the HTTP client."""
        self._client.close()

    def __enter__(self) -> "HttpClient":
        return self

    def __exit__(self, *args: Any) -> None:
        self.close()



class AsyncHttpClient:
    """Async HTTP client for Dfns API requests."""

    def __init__(self, config: DfnsClientConfig):
        self.config = config
        self._client = httpx.AsyncClient(
            base_url=config.base_url,
            timeout=30.0,
        )

    def _build_headers(self, user_action_token: Optional[str] = None) -> dict[str, str]:
        """Build request headers."""
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.config.auth_token}",
            **self.config.headers,
        }

        if user_action_token:
            headers["X-DFNS-USERACTION"] = user_action_token

        return headers

    def _build_url(
        self,
        path: str,
        path_params: Optional[dict[str, Any]] = None,
        query_params: Optional[dict[str, Any]] = None,
    ) -> str:
        """Build the full URL with path and query parameters."""
        url = path

        if path_params:
            for key, value in path_params.items():
                url = url.replace(f"{{{key}}}", str(value))

        if query_params:
            filtered_params = {k: v for k, v in query_params.items() if v is not None}
            if filtered_params:
                url = f"{url}?{urlencode(filtered_params)}"

        return url

    def _handle_response(self, response: httpx.Response) -> Any:
        """Handle API response and raise errors if needed."""
        if response.status_code >= 400:
            try:
                error_data = response.json()
                raise DfnsError(
                    message=error_data.get("message", "Unknown error"),
                    status_code=response.status_code,
                    error_code=error_data.get("error"),
                    details=error_data.get("details"),
                )
            except json.JSONDecodeError:
                raise DfnsError(
                    message=response.text or "Unknown error",
                    status_code=response.status_code,
                )

        if response.status_code == 204 or not response.content:
            return None

        return response.json()

    async def _get_user_action_token(
        self,
        method: str,
        path: str,
        body: Optional[Any] = None,
    ) -> str:
        """
        Get a user action token by creating and signing a challenge.

        Args:
            method: HTTP method.
            path: Request path.
            body: Request body.

        Returns:
            The user action token to include in the request header.

        Raises:
            DfnsError: If no signer is configured or signing fails.
        """
        if not self.config.signer:
            raise DfnsError(
                message="Signer required for this operation. Configure a signer in DfnsClientConfig.",
                status_code=None,
                error_code="SIGNER_REQUIRED",
            )

        # Step 1: Create user action challenge
        challenge_body = {
            "userActionPayload": json.dumps(body, separators=(",", ":")) if body else "",
            "userActionHttpMethod": method,
            "userActionHttpPath": path,
            "userActionServerKind": "Api",
        }

        challenge_response = await self._client.request(
            method="POST",
            url="/auth/action/init",
            headers=self._build_headers(),
            json=challenge_body,
        )
        challenge = self._handle_response(challenge_response)

        # Step 2: Sign the challenge
        assertion = self.config.signer.sign(challenge)

        # Step 3: Submit signed challenge to get user action token
        signature_body = {
            "challengeIdentifier": challenge["challengeIdentifier"],
            "firstFactor": assertion,
        }

        signature_response = await self._client.request(
            method="POST",
            url="/auth/action",
            headers=self._build_headers(),
            json=signature_body,
        )
        result = self._handle_response(signature_response)

        return result["userAction"]

    async def request(
        self,
        method: str,
        path: str,
        path_params: Optional[dict[str, Any]] = None,
        query_params: Optional[dict[str, Any]] = None,
        body: Optional[Any] = None,
        requires_signature: bool = False,
    ) -> Any:
        """Make an async HTTP request to the API."""
        url = self._build_url(path, path_params, query_params)

        # Get user action token if required
        user_action_token = None
        if requires_signature:
            # Use the path with params substituted for signing
            signing_path = path
            if path_params:
                for key, value in path_params.items():
                    signing_path = signing_path.replace(f"{{{key}}}", str(value))
            user_action_token = await self._get_user_action_token(method, signing_path, body)

        headers = self._build_headers(user_action_token)

        response = await self._client.request(
            method=method,
            url=url,
            headers=headers,
            json=body if body is not None else None,
        )

        return self._handle_response(response)

    async def close(self) -> None:
        """Close the HTTP client."""
        await self._client.aclose()

    async def __aenter__(self) -> "AsyncHttpClient":
        return self

    async def __aexit__(self, *args: Any) -> None:
        await self.close()



"""Types for the agreements domain."""

from typing import Any, Literal, NotRequired, Optional, TypedDict, Union

class GetLatestUnacceptedAgreementResponse(TypedDict, total=False):
    """getLatestUnacceptedAgreement response."""

    latest_agreement: Any

class GetLatestUnacceptedAgreementQuery(TypedDict, total=False):
    """getLatestUnacceptedAgreement query parameters."""

    agreement_type: Literal["PrivacyPolicy", "TermsAndConditions", "UniswapTermsOfService", "UniswapPrivacyPolicy"]

class RecordAgreementAcceptanceResponse(TypedDict, total=False):
    """recordAgreementAcceptance response."""

    agreement_id: str
    user_id: str
    date_accepted: str



"""Types for the networks domain."""

from typing import Any, Literal, NotRequired, Optional, TypedDict, Union

class EstimateFeesQuery(TypedDict, total=False):
    """estimateFees query parameters."""

    network: Literal["Bitcoin", "BitcoinSignet", "BitcoinTestnet3", "Adi", "AdiTestnet", "AdiTestnetAb", "ArbitrumOne", "ArbitrumGoerli", "ArbitrumSepolia", "ArcTestnet", "AvalancheC", "AvalancheCFuji", "Base", "BaseGoerli", "BaseSepolia", "Bob", "BobSepolia", "Bsc", "BscTestnet", "Berachain", "BerachainBArtio", "BerachainBepolia", "Celo", "CeloAlfajores", "Codex", "CodexSepolia", "Ethereum", "EthereumClassic", "EthereumClassicMordor", "EthereumGoerli", "EthereumSepolia", "EthereumHolesky", "EthereumHoodi", "FantomOpera", "FantomTestnet", "FlareC", "FlareCCoston2", "FlowEvm", "FlowEvmTestnet", "Ink", "InkSepolia", "Optimism", "OptimismGoerli", "OptimismSepolia", "Plasma", "PlasmaTestnet", "Plume", "PlumeSepolia", "Polygon", "PolygonAmoy", "PolygonMumbai", "Race", "RaceSepolia", "Sonic", "SonicTestnet", "TempoAndantino", "Tsc", "TscTestnet1"]

class ReadContractResponse(TypedDict, total=False):
    """readContract response."""

    kind: Literal["Evm"]
    data: str

class GetCantonValidatorResponse(TypedDict, total=False):
    """getCantonValidator response."""

    id: str
    org_id: str
    network: Literal["Canton", "CantonDevnet", "CantonTestnet"]
    name: NotRequired[str]
    kind: Literal["Shared", "Custom"]
    date_created: str
    party_hint: str

class UpdateCantonValidatorRequest(TypedDict, total=False):
    """updateCantonValidator request body."""

    name: NotRequired[str]
    validator: NotRequired[TypedDict]
    ledger: NotRequired[TypedDict]

class UpdateCantonValidatorResponse(TypedDict, total=False):
    """updateCantonValidator response."""

    id: str
    org_id: str
    network: Literal["Canton", "CantonDevnet", "CantonTestnet"]
    name: NotRequired[str]
    kind: Literal["Shared", "Custom"]
    date_created: str
    party_hint: str

class DeleteCantonValidatorResponse(TypedDict, total=False):
    """deleteCantonValidator response."""

    id: str
    org_id: str
    network: Literal["Canton", "CantonDevnet", "CantonTestnet"]
    name: NotRequired[str]
    kind: Literal["Shared", "Custom"]
    date_created: str
    party_hint: str

class ListCantonValidatorsResponse(TypedDict, total=False):
    """listCantonValidators response."""

    items: list[TypedDict]
    next_page_token: NotRequired[str]

class ListCantonValidatorsQuery(TypedDict, total=False):
    """listCantonValidators query parameters."""

    limit: NotRequired[int]
    pagination_token: NotRequired[str]

class CreateCantonValidatorResponse(TypedDict, total=False):
    """createCantonValidator response."""

    id: str
    org_id: str
    network: Literal["Canton", "CantonDevnet", "CantonTestnet"]
    name: NotRequired[str]
    kind: Literal["Shared", "Custom"]
    date_created: str
    party_hint: str



"""Types for the signers domain."""

from typing import Any, Literal, NotRequired, Optional, TypedDict, Union

class ListKeyStoresResponse(TypedDict, total=False):
    """listKeyStores response."""

    items: list[TypedDict]

class ListSignersResponse(TypedDict, total=False):
    """listSigners response."""

    clusters: list[TypedDict]



"""Client for the signers domain."""

from typing import Any, Literal, Optional, TypedDict, Union

from ..._internal import HttpClient
from . import types as T


class SignersClient:
    """Client for signers operations."""

    def __init__(self, http_client: HttpClient):
        self._http = http_client

    def list_key_stores(self) -> T.ListKeyStoresResponse:
        """
        List Key Stores.

        Returns:
            T.ListKeyStoresResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/key-stores",
            path_params={},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def list_signers(self) -> T.ListSignersResponse:
        """
        List Signers.

        Returns:
            T.ListSignersResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/signers",
            path_params={},
            query_params=None,
            body=None,
            requires_signature=False,
        )



"""Delegated client for the allocations domain."""

import json
from typing import Any, Literal, Optional, TypedDict, Union

from ..._internal import HttpClient
from ...base_auth_api import (
    BaseAuthApi,
    SignUserActionChallengeRequest,
    UserActionChallengeResponse,
)
from . import types as T


class DelegatedAllocationsClient:
    """
    Delegated client for allocations operations.

    This client separates user action signing into _init() and _complete() method pairs,
    allowing external systems to handle the signing process.
    """

    def __init__(self, http_client: HttpClient):
        self._http = http_client

    def list_allocations(self, query: Optional[T.ListAllocationsQuery] = None) -> T.ListAllocationsResponse:
        """
        List Allocations.

        Args:
        query: Query parameters.

        Returns:
            T.ListAllocationsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/allocations",
            path_params={},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def create_allocation_init(self, body: dict[str, Any]) -> UserActionChallengeResponse:
        """
        Initialize Create Allocation.

        Creates a user action challenge for external signing.

        Args:
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/allocations"
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def create_allocation_complete(self, body: dict[str, Any], signed_challenge: SignUserActionChallengeRequest) -> T.CreateAllocationResponse:
        """
        Complete Create Allocation.

        Submits the signed challenge and makes the API request.

        Args:
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.CreateAllocationResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/allocations",
            path_params={},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def list_allocation_actions(self, allocation_id: str, query: Optional[T.ListAllocationActionsQuery] = None) -> T.ListAllocationActionsResponse:
        """
        List Allocation Actions.

        Retrieve the list of actions for a specific allocation.

        Args:
        allocation_id: Unique identifier for the allocation investment.
        query: Query parameters.

        Returns:
            T.ListAllocationActionsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/allocations/{allocationId}/actions",
            path_params={"allocationId": allocation_id},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def create_allocation_action_init(self, allocation_id: str, body: T.CreateAllocationActionRequest) -> UserActionChallengeResponse:
        """
        Initialize Create Allocation Action.

        Creates a user action challenge for external signing.

        Args:
        allocation_id: Unique identifier for the allocation investment.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/allocations/{allocationId}/actions"
        path = path.replace("{allocationId}", str(allocation_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def create_allocation_action_complete(self, allocation_id: str, body: T.CreateAllocationActionRequest, signed_challenge: SignUserActionChallengeRequest) -> T.CreateAllocationActionResponse:
        """
        Complete Create Allocation Action.

        Submits the signed challenge and makes the API request.

        Args:
        allocation_id: Unique identifier for the allocation investment.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.CreateAllocationActionResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/allocations/{allocationId}/actions",
            path_params={"allocationId": allocation_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def get_allocation(self, allocation_id: str) -> T.GetAllocationResponse:
        """
        Get Allocation.

        Retrieve the details of a specific allocation.

        Args:
        allocation_id: Unique identifier for the allocation investment.

        Returns:
            T.GetAllocationResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/allocations/{allocationId}",
            path_params={"allocationId": allocation_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )



"""Client for the permissions domain."""

from typing import Any, Literal, Optional, TypedDict, Union

from ..._internal import HttpClient
from . import types as T


class PermissionsClient:
    """Client for permissions operations."""

    def __init__(self, http_client: HttpClient):
        self._http = http_client

    def list_permission_assignments(self, permission_id: str) -> T.ListPermissionAssignmentsResponse:
        """
        List Permission Assignments.

        Retrieves a list of permission assignments (success) or gives a reason why it's not possible (failure).

        Args:
        permission_id: Path parameter.

        Returns:
            T.ListPermissionAssignmentsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/permissions/{permissionId}/assignments",
            path_params={"permissionId": permission_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def assign_permission(self, permission_id: str, body: T.AssignPermissionRequest) -> T.AssignPermissionResponse:
        """
        Assign Permission.

        Creates a permission that allows certain specified operations to be executed. 
  
  Response is either the permission object itself (success) or a reason why it was not possible to create the permission (failure).

        Args:
        permission_id: Path parameter.
        body: Request body.

        Returns:
            T.AssignPermissionResponse: The API response.
        """
        return self._http.request(
            method="POST",
            path="/permissions/{permissionId}/assignments",
            path_params={"permissionId": permission_id},
            query_params=None,
            body=body,
            requires_signature=True,
        )

    def revoke_permission(self, permission_id: str, assignment_id: str, query: Optional[T.RevokePermissionQuery] = None) -> None:
        """
        Revoke Permission.

        Revokes a permission assignment (success) or gives reason why it’s not possible (failure).

        Args:
        permission_id: Path parameter.
        assignment_id: Path parameter.
        query: Query parameters.
        """
        return self._http.request(
            method="DELETE",
            path="/permissions/{permissionId}/assignments/{assignmentId}",
            path_params={"permissionId": permission_id, "assignmentId": assignment_id},
            query_params=query,
            body=None,
            requires_signature=True,
        )

    def delete_permission(self, permission_id: str, body: T.DeletePermissionRequest) -> T.DeletePermissionResponse:
        """
        Delete Permission.

        Delete a specific Permission.

        Args:
        permission_id: Path parameter.
        body: Request body.

        Returns:
            T.DeletePermissionResponse: The API response.
        """
        return self._http.request(
            method="PUT",
            path="/permissions/{permissionId}/archive",
            path_params={"permissionId": permission_id},
            query_params=None,
            body=body,
            requires_signature=True,
        )

    def list_permissions(self, query: Optional[T.ListPermissionsQuery] = None) -> T.ListPermissionsResponse:
        """
        List Permissions.

        Retrieves a list of permissions (success) or gives a reason why it's not possible (failure).

        Args:
        query: Query parameters.

        Returns:
            T.ListPermissionsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/permissions",
            path_params={},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def create_permission(self, body: T.CreatePermissionRequest) -> T.CreatePermissionResponse:
        """
        Create Permission.

        Creates a permission that allows certain specified operations to be executed.

        Args:
        body: Request body.

        Returns:
            T.CreatePermissionResponse: The API response.
        """
        return self._http.request(
            method="POST",
            path="/permissions",
            path_params={},
            query_params=None,
            body=body,
            requires_signature=True,
        )

    def get_permission(self, permission_id: str) -> T.GetPermissionResponse:
        """
        Get Permission.

        Retrieves a specific permission (success) or gives a reason why it's not possible (failure).

        Args:
        permission_id: Path parameter.

        Returns:
            T.GetPermissionResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/permissions/{permissionId}",
            path_params={"permissionId": permission_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def update_permission(self, permission_id: str, body: T.UpdatePermissionRequest) -> T.UpdatePermissionResponse:
        """
        Update Permission.

        Updates an existing permission. Response either returns the updated permission (success) or the reason why it was not possible to update (failure).

        Args:
        permission_id: Path parameter.
        body: Request body.

        Returns:
            T.UpdatePermissionResponse: The API response.
        """
        return self._http.request(
            method="PUT",
            path="/permissions/{permissionId}",
            path_params={"permissionId": permission_id},
            query_params=None,
            body=body,
            requires_signature=True,
        )



"""Dfns Python SDK - Auto-generated from OpenAPI specification."""

try:
    from ._version import __version__
except ImportError:
    __version__ = "0.0.0-dev"

from .client import DfnsClient
from .delegated_client import DfnsDelegatedClient
from .types import DfnsClientConfig, DfnsDelegatedClientConfig, DfnsError
from .auth import Signer, KeySigner
from .base_auth_api import (
    BaseAuthApi,
    UserActionChallengeResponse,
    SignUserActionChallengeRequest,
)

__all__ = [
    "__version__",
    "DfnsClient",
    "DfnsDelegatedClient",
    "DfnsClientConfig",
    "DfnsDelegatedClientConfig",
    "DfnsError",
    "Signer",
    "KeySigner",
    "BaseAuthApi",
    "UserActionChallengeResponse",
    "SignUserActionChallengeRequest",
]



"""Delegated client for the agreements domain."""

import json
from typing import Any, Literal, Optional, TypedDict, Union

from ..._internal import HttpClient
from ...base_auth_api import (
    BaseAuthApi,
    SignUserActionChallengeRequest,
    UserActionChallengeResponse,
)
from . import types as T


class DelegatedAgreementsClient:
    """
    Delegated client for agreements operations.

    This client separates user action signing into _init() and _complete() method pairs,
    allowing external systems to handle the signing process.
    """

    def __init__(self, http_client: HttpClient):
        self._http = http_client

    def get_latest_unaccepted_agreement(self, query: T.GetLatestUnacceptedAgreementQuery) -> T.GetLatestUnacceptedAgreementResponse:
        """
        Get Latest Unaccepted Agreement.

        Get the latest unaccepted agreement for a specific agreement type

        Args:
        query: Query parameters.

        Returns:
            T.GetLatestUnacceptedAgreementResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/agreements/latest-unaccepted",
            path_params={},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def record_agreement_acceptance_init(self, agreement_id: str) -> UserActionChallengeResponse:
        """
        Initialize Record Agreement Acceptance.

        Creates a user action challenge for external signing.

        Args:
        agreement_id: Path parameter.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/agreements/{agreementId}/accept"
        path = path.replace("{agreementId}", str(agreement_id))
        payload = ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def record_agreement_acceptance_complete(self, agreement_id: str, signed_challenge: SignUserActionChallengeRequest) -> T.RecordAgreementAcceptanceResponse:
        """
        Complete Record Agreement Acceptance.

        Submits the signed challenge and makes the API request.

        Args:
        agreement_id: Path parameter.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.RecordAgreementAcceptanceResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/agreements/{agreementId}/accept",
            path_params={"agreementId": agreement_id},
            query_params=None,
            body=None,
            user_action=user_action_token,
        )



"""Webhooks domain module."""

from .client import WebhooksClient
from .delegated_client import DelegatedWebhooksClient
from . import types

__all__ = ["WebhooksClient", "DelegatedWebhooksClient", "types"]



"""FeeSponsors domain module."""

from .client import FeeSponsorsClient
from .delegated_client import DelegatedFeeSponsorsClient
from . import types

__all__ = ["FeeSponsorsClient", "DelegatedFeeSponsorsClient", "types"]



"""Client for the fee_sponsors domain."""

from typing import Any, Literal, Optional, TypedDict, Union

from ..._internal import HttpClient
from . import types as T


class FeeSponsorsClient:
    """Client for fee_sponsors operations."""

    def __init__(self, http_client: HttpClient):
        self._http = http_client

    def list_fee_sponsors(self, query: Optional[T.ListFeeSponsorsQuery] = None) -> T.ListFeeSponsorsResponse:
        """
        List Fee Sponsors.

        Retrieves all Fee Sponsors configured in your organization.

        Args:
        query: Query parameters.

        Returns:
            T.ListFeeSponsorsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/fee-sponsors",
            path_params={},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def create_fee_sponsor(self, body: T.CreateFeeSponsorRequest) -> T.CreateFeeSponsorResponse:
        """
        Create Fee Sponsor.

        Creates a new `FeeSponsor` associated with a sponsor wallet. Returns a new fee sponsor entity with the `id` to be used when making a transfer.

        Args:
        body: Request body.

        Returns:
            T.CreateFeeSponsorResponse: The API response.
        """
        return self._http.request(
            method="POST",
            path="/fee-sponsors",
            path_params={},
            query_params=None,
            body=body,
            requires_signature=True,
        )

    def get_fee_sponsor(self, fee_sponsor_id: str) -> T.GetFeeSponsorResponse:
        """
        Get Fee Sponsor.

        Retrieve a Fee Sponsor information by ID.

        Args:
        fee_sponsor_id: Which Fee Sponsor you wish to retrieve.

        Returns:
            T.GetFeeSponsorResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/fee-sponsors/{feeSponsorId}",
            path_params={"feeSponsorId": fee_sponsor_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def delete_fee_sponsor(self, fee_sponsor_id: str) -> T.DeleteFeeSponsorResponse:
        """
        Delete Fee Sponsor.

        Delete a Fee Sponsor. This action is irreversible. The fee sponsor won't be able to be used anymore when making a transfer.

        Args:
        fee_sponsor_id: Which Fee Sponsor you wish to delete.

        Returns:
            T.DeleteFeeSponsorResponse: The API response.
        """
        return self._http.request(
            method="DELETE",
            path="/fee-sponsors/{feeSponsorId}",
            path_params={"feeSponsorId": fee_sponsor_id},
            query_params=None,
            body=None,
            requires_signature=True,
        )

    def deactivate_fee_sponsor(self, fee_sponsor_id: str) -> T.DeactivateFeeSponsorResponse:
        """
        Deactivate Fee Sponsor.

        Deactivate a Fee Sponsor: The fee sponsor won't be able to be used anymore when making a transfer.

        Args:
        fee_sponsor_id: Which Fee Sponsor you wish to deactivate.

        Returns:
            T.DeactivateFeeSponsorResponse: The API response.
        """
        return self._http.request(
            method="PUT",
            path="/fee-sponsors/{feeSponsorId}/deactivate",
            path_params={"feeSponsorId": fee_sponsor_id},
            query_params=None,
            body=None,
            requires_signature=True,
        )

    def activate_fee_sponsor(self, fee_sponsor_id: str) -> T.ActivateFeeSponsorResponse:
        """
        Activate Fee Sponsor.

        Activate a Fee Sponsor: The fee sponsor can be used when making a transfer.

        Args:
        fee_sponsor_id: Which Fee Sponsor you wish to activate.

        Returns:
            T.ActivateFeeSponsorResponse: The API response.
        """
        return self._http.request(
            method="PUT",
            path="/fee-sponsors/{feeSponsorId}/activate",
            path_params={"feeSponsorId": fee_sponsor_id},
            query_params=None,
            body=None,
            requires_signature=True,
        )

    def list_sponsored_fees(self, fee_sponsor_id: str, query: Optional[T.ListSponsoredFeesQuery] = None) -> T.ListSponsoredFeesResponse:
        """
        List Sponsored Fees.

        Retrieves all fees paid by the specific Fee Sponsor.

        Args:
        fee_sponsor_id: Fee Sponsor to retrieve the fees from.
        query: Query parameters.

        Returns:
            T.ListSponsoredFeesResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/fee-sponsors/{feeSponsorId}/fees",
            path_params={"feeSponsorId": fee_sponsor_id},
            query_params=query,
            body=None,
            requires_signature=False,
        )



"""Authentication utilities for the Dfns SDK."""

import base64
import hashlib
import json
from typing import Any, Optional, Protocol, TypedDict

from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import ec, ed25519, padding


class UserActionChallenge(TypedDict):
    """Challenge returned by the server for user action signing."""

    challenge: str
    challengeIdentifier: str
    supportedCredentialKinds: list[dict[str, Any]]
    allowCredentials: dict[str, Any]
    rp: dict[str, str]
    externalAuthenticationUrl: str


class CredentialAssertion(TypedDict):
    """Credential assertion for signing."""

    kind: str
    credentialAssertion: dict[str, str]


class Signer(Protocol):
    """Protocol for signing user action challenges."""

    def sign(self, challenge: UserActionChallenge) -> CredentialAssertion:
        """
        Sign a user action challenge.

        Args:
            challenge: The challenge from the server.

        Returns:
            The credential assertion to submit to the server.
        """
        ...


def base64url_encode(data: bytes) -> str:
    """Encode bytes to base64url string without padding."""
    return base64.urlsafe_b64encode(data).rstrip(b"=").decode("ascii")


def base64url_decode(data: str) -> bytes:
    """Decode base64url string to bytes."""
    padding = 4 - len(data) % 4
    if padding != 4:
        data += "=" * padding
    return base64.urlsafe_b64decode(data)


class KeySigner:
    """
    Signer implementation using a private key credential.

    Supports Ed25519, ECDSA (secp256k1, P-256), and RSA keys.

    Example:
        >>> from dfns_sdk.auth import KeySigner
        >>> signer = KeySigner(
        ...     credential_id="cr-xxx-xxx",
        ...     private_key=open("private_key.pem").read(),
        ...     app_origin="https://app.dfns.io",
        ... )
    """

    def __init__(
        self,
        credential_id: str,
        private_key: str,
        app_origin: str = "https://app.dfns.io",
    ):
        """
        Initialize the key signer.

        Args:
            credential_id: The credential ID (cr-xxx-xxx format).
            private_key: The private key in PEM format.
            app_origin: The application origin for the client data.
        """
        self.credential_id = credential_id
        self.app_origin = app_origin
        self._private_key = serialization.load_pem_private_key(
            private_key.encode() if isinstance(private_key, str) else private_key,
            password=None,
        )

    def sign(self, challenge: UserActionChallenge) -> CredentialAssertion:
        """
        Sign a user action challenge.

        Args:
            challenge: The challenge from the server.

        Returns:
            The credential assertion to submit to the server.
        """
        client_data = {
            "type": "key.get",
            "challenge": challenge["challenge"],
            "origin": self.app_origin,
            "crossOrigin": False,
        }
        client_data_json = json.dumps(client_data, separators=(",", ":"))
        client_data_bytes = client_data_json.encode("utf-8")

        signature = self._sign_bytes(client_data_bytes)

        return {
            "kind": "Key",
            "credentialAssertion": {
                "credId": self.credential_id,
                "clientData": base64url_encode(client_data_bytes),
                "signature": base64url_encode(signature),
            },
        }

    def _sign_bytes(self, data: bytes) -> bytes:
        """Sign bytes with the private key."""
        if isinstance(self._private_key, ed25519.Ed25519PrivateKey):
            return self._private_key.sign(data)
        elif isinstance(self._private_key, ec.EllipticCurvePrivateKey):
            return self._private_key.sign(data, ec.ECDSA(hashes.SHA256()))
        else:
            # RSA
            return self._private_key.sign(
                data,
                padding.PKCS1v15(),
                hashes.SHA256(),
            )



"""Types for the allocations domain."""

from typing import Any, Literal, NotRequired, Optional, TypedDict, Union

class ListAllocationsResponse(TypedDict, total=False):
    """listAllocations response."""

    items: list[TypedDict]
    next_page_token: NotRequired[str]

class ListAllocationsQuery(TypedDict, total=False):
    """listAllocations query parameters."""

    limit: NotRequired[int]
    pagination_token: NotRequired[str]

class CreateAllocationResponse(TypedDict, total=False):
    """createAllocation response."""

    id: str
    wallet_id: str
    protocol: Literal["0fns"]
    amount: TypedDict
    rewards: TypedDict
    date_created: str
    actions: list[TypedDict]

class ListAllocationActionsResponse(TypedDict, total=False):
    """listAllocationActions response."""

    items: list[TypedDict]
    next_page_token: NotRequired[str]

class ListAllocationActionsQuery(TypedDict, total=False):
    """listAllocationActions query parameters."""

    limit: NotRequired[int]
    pagination_token: NotRequired[str]

class CreateAllocationActionRequest(TypedDict, total=False):
    """createAllocationAction request body."""

    kind: Literal["Deposit", "Withdraw"]
    external_id: NotRequired[str]
    source_asset: TypedDict
    target_asset: TypedDict
    slippage_bps: float

class CreateAllocationActionResponse(TypedDict, total=False):
    """createAllocationAction response."""

    id: str
    wallet_id: str
    protocol: Literal["0fns"]
    amount: TypedDict
    rewards: TypedDict
    date_created: str
    actions: list[TypedDict]

class GetAllocationResponse(TypedDict, total=False):
    """getAllocation response."""

    id: str
    wallet_id: str
    protocol: Literal["0fns"]
    amount: TypedDict
    rewards: TypedDict
    date_created: str
    actions: list[TypedDict]



"""Delegated client for the keys domain."""

import json
from typing import Any, Literal, Optional, TypedDict, Union

from ..._internal import HttpClient
from ...base_auth_api import (
    BaseAuthApi,
    SignUserActionChallengeRequest,
    UserActionChallengeResponse,
)
from . import types as T


class DelegatedKeysClient:
    """
    Delegated client for keys operations.

    This client separates user action signing into _init() and _complete() method pairs,
    allowing external systems to handle the signing process.
    """

    def __init__(self, http_client: HttpClient):
        self._http = http_client

    def list_keys(self, query: Optional[T.ListKeysQuery] = None) -> T.ListKeysResponse:
        """
        List Keys.

        Retrieve all keys registered for your organization.

        Args:
        query: Query parameters.

        Returns:
            T.ListKeysResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/keys",
            path_params={},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def create_key_init(self, body: T.CreateKeyRequest) -> UserActionChallengeResponse:
        """
        Initialize Create Key.

        Creates a user action challenge for external signing.

        Args:
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/keys"
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def create_key_complete(self, body: T.CreateKeyRequest, signed_challenge: SignUserActionChallengeRequest) -> T.CreateKeyResponse:
        """
        Complete Create Key.

        Submits the signed challenge and makes the API request.

        Args:
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.CreateKeyResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/keys",
            path_params={},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def delegate_key_init(self, key_id: str, body: T.DelegateKeyRequest) -> UserActionChallengeResponse:
        """
        Initialize Delegate Key.

        Creates a user action challenge for external signing.

        Args:
        key_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/keys/{keyId}/delegate"
        path = path.replace("{keyId}", str(key_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def delegate_key_complete(self, key_id: str, body: T.DelegateKeyRequest, signed_challenge: SignUserActionChallengeRequest) -> T.DelegateKeyResponse:
        """
        Complete Delegate Key.

        Submits the signed challenge and makes the API request.

        Args:
        key_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.DelegateKeyResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/keys/{keyId}/delegate",
            path_params={"keyId": key_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def get_key(self, key_id: str) -> T.GetKeyResponse:
        """
        Get Key.

        Retrieves a key information by its ID.

        Args:
        key_id: Path parameter.

        Returns:
            T.GetKeyResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/keys/{keyId}",
            path_params={"keyId": key_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def update_key_init(self, key_id: str, body: T.UpdateKeyRequest) -> UserActionChallengeResponse:
        """
        Initialize Update Key.

        Creates a user action challenge for external signing.

        Args:
        key_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/keys/{keyId}"
        path = path.replace("{keyId}", str(key_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="PUT",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def update_key_complete(self, key_id: str, body: T.UpdateKeyRequest, signed_challenge: SignUserActionChallengeRequest) -> T.UpdateKeyResponse:
        """
        Complete Update Key.

        Submits the signed challenge and makes the API request.

        Args:
        key_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.UpdateKeyResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="PUT",
            path="/keys/{keyId}",
            path_params={"keyId": key_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def delete_key_init(self, key_id: str) -> UserActionChallengeResponse:
        """
        Initialize Delete Key.

        Creates a user action challenge for external signing.

        Args:
        key_id: Path parameter.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/keys/{keyId}"
        path = path.replace("{keyId}", str(key_id))
        payload = ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="DELETE",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def delete_key_complete(self, key_id: str, signed_challenge: SignUserActionChallengeRequest) -> T.DeleteKeyResponse:
        """
        Complete Delete Key.

        Submits the signed challenge and makes the API request.

        Args:
        key_id: Path parameter.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.DeleteKeyResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="DELETE",
            path="/keys/{keyId}",
            path_params={"keyId": key_id},
            query_params=None,
            body=None,
            user_action=user_action_token,
        )

    def derive_key_init(self, key_id: str, body: T.DeriveKeyRequest) -> UserActionChallengeResponse:
        """
        Initialize Derive Key.

        Creates a user action challenge for external signing.

        Args:
        key_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/keys/{keyId}/derive"
        path = path.replace("{keyId}", str(key_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def derive_key_complete(self, key_id: str, body: T.DeriveKeyRequest, signed_challenge: SignUserActionChallengeRequest) -> T.DeriveKeyResponse:
        """
        Complete Derive Key.

        Submits the signed challenge and makes the API request.

        Args:
        key_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.DeriveKeyResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/keys/{keyId}/derive",
            path_params={"keyId": key_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def export_key_init(self, key_id: str, body: T.ExportKeyRequest) -> UserActionChallengeResponse:
        """
        Initialize Export Key.

        Creates a user action challenge for external signing.

        Args:
        key_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/keys/{keyId}/export"
        path = path.replace("{keyId}", str(key_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def export_key_complete(self, key_id: str, body: T.ExportKeyRequest, signed_challenge: SignUserActionChallengeRequest) -> T.ExportKeyResponse:
        """
        Complete Export Key.

        Submits the signed challenge and makes the API request.

        Args:
        key_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.ExportKeyResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/keys/{keyId}/export",
            path_params={"keyId": key_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def list_signatures(self, key_id: str, query: Optional[T.ListSignaturesQuery] = None) -> T.ListSignaturesResponse:
        """
        List Signatures.

        List all signature requests for a key.

        Args:
        key_id: Path parameter.
        query: Query parameters.

        Returns:
            T.ListSignaturesResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/keys/{keyId}/signatures",
            path_params={"keyId": key_id},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def generate_signature_init(self, key_id: str, body: dict[str, Any]) -> UserActionChallengeResponse:
        """
        Initialize Generate Signature.

        Creates a user action challenge for external signing.

        Args:
        key_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/keys/{keyId}/signatures"
        path = path.replace("{keyId}", str(key_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def generate_signature_complete(self, key_id: str, body: dict[str, Any], signed_challenge: SignUserActionChallengeRequest) -> T.GenerateSignatureResponse:
        """
        Complete Generate Signature.

        Submits the signed challenge and makes the API request.

        Args:
        key_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.GenerateSignatureResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/keys/{keyId}/signatures",
            path_params={"keyId": key_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def get_signature(self, key_id: str, signature_id: str) -> T.GetSignatureResponse:
        """
        Get Signature.

        Retrieve a signature request details.

        Args:
        key_id: Path parameter.
        signature_id: Path parameter.

        Returns:
            T.GetSignatureResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/keys/{keyId}/signatures/{signatureId}",
            path_params={"keyId": key_id, "signatureId": signature_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def import_key_init(self, body: T.ImportKeyRequest) -> UserActionChallengeResponse:
        """
        Initialize Import Key.

        Creates a user action challenge for external signing.

        Args:
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/keys/import"
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def import_key_complete(self, body: T.ImportKeyRequest, signed_challenge: SignUserActionChallengeRequest) -> T.ImportKeyResponse:
        """
        Complete Import Key.

        Submits the signed challenge and makes the API request.

        Args:
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.ImportKeyResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/keys/import",
            path_params={},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )



"""Types for the exchanges domain."""

from typing import Any, Literal, NotRequired, Optional, TypedDict, Union

class GetExchangeResponse(TypedDict, total=False):
    """getExchange response."""

    id: str
    name: NotRequired[str]
    kind: Literal["Binance", "Kraken", "CoinbaseApp", "CoinbasePrime"]
    date_created: str

class DeleteExchangeResponse(TypedDict, total=False):
    """deleteExchange response."""

    deleted: Literal[True]

class ListExchangesResponse(TypedDict, total=False):
    """listExchanges response."""

    items: list[TypedDict]
    next_page_token: NotRequired[str]

class ListExchangesQuery(TypedDict, total=False):
    """listExchanges query parameters."""

    limit: NotRequired[int]
    pagination_token: NotRequired[str]

class CreateExchangeRequest(TypedDict, total=False):
    """createExchange request body."""

    name: NotRequired[str]
    kind: Literal["Binance", "Kraken", "CoinbaseApp", "CoinbasePrime"]
    read_configuration: TypedDict
    write_configuration: TypedDict

class CreateExchangeResponse(TypedDict, total=False):
    """createExchange response."""

    id: str
    name: NotRequired[str]
    kind: Literal["Binance", "Kraken", "CoinbaseApp", "CoinbasePrime"]
    date_created: str

class ListAccountsResponse(TypedDict, total=False):
    """listAccounts response."""

    items: list[TypedDict]
    next_page_token: NotRequired[str]

class ListAccountsQuery(TypedDict, total=False):
    """listAccounts query parameters."""

    limit: NotRequired[int]
    pagination_token: NotRequired[str]

class ListAccountAssetsResponse(TypedDict, total=False):
    """listAccountAssets response."""

    items: list[TypedDict]
    next_page_token: NotRequired[str]

class ListAccountAssetsQuery(TypedDict, total=False):
    """listAccountAssets query parameters."""

    limit: NotRequired[int]
    pagination_token: NotRequired[str]

class CreateExchangeDepositResponse(TypedDict, total=False):
    """createExchangeDeposit response."""

    id: str
    exchange_id: str
    account_id: str
    transfer_id: NotRequired[str]
    exchange_reference: NotRequired[str]
    kind: Literal["Withdrawal", "Deposit"]
    wallet_id: str
    requester: TypedDict
    request_body: TypedDict
    date_created: str

class CreateExchangeWithdrawalResponse(TypedDict, total=False):
    """createExchangeWithdrawal response."""

    id: str
    exchange_id: str
    account_id: str
    transfer_id: NotRequired[str]
    exchange_reference: NotRequired[str]
    kind: Literal["Withdrawal", "Deposit"]
    wallet_id: str
    requester: TypedDict
    request_body: TypedDict
    date_created: str



"""Generated domain clients."""

from .auth import AuthClient, DelegatedAuthClient
from .exchanges import ExchangesClient, DelegatedExchangesClient
from .fee_sponsors import FeeSponsorsClient, DelegatedFeeSponsorsClient
from .keys import KeysClient, DelegatedKeysClient
from .networks import NetworksClient, DelegatedNetworksClient
from .permissions import PermissionsClient, DelegatedPermissionsClient
from .policies import PoliciesClient, DelegatedPoliciesClient
from .signers import SignersClient, DelegatedSignersClient
from .staking import StakingClient, DelegatedStakingClient
from .wallets import WalletsClient, DelegatedWalletsClient
from .webhooks import WebhooksClient, DelegatedWebhooksClient
from .swaps import SwapsClient, DelegatedSwapsClient
from .agreements import AgreementsClient, DelegatedAgreementsClient
from .allocations import AllocationsClient, DelegatedAllocationsClient

__all__ = [
    "AuthClient",
    "DelegatedAuthClient",
    "ExchangesClient",
    "DelegatedExchangesClient",
    "FeeSponsorsClient",
    "DelegatedFeeSponsorsClient",
    "KeysClient",
    "DelegatedKeysClient",
    "NetworksClient",
    "DelegatedNetworksClient",
    "PermissionsClient",
    "DelegatedPermissionsClient",
    "PoliciesClient",
    "DelegatedPoliciesClient",
    "SignersClient",
    "DelegatedSignersClient",
    "StakingClient",
    "DelegatedStakingClient",
    "WalletsClient",
    "DelegatedWalletsClient",
    "WebhooksClient",
    "DelegatedWebhooksClient",
    "SwapsClient",
    "DelegatedSwapsClient",
    "AgreementsClient",
    "DelegatedAgreementsClient",
    "AllocationsClient",
    "DelegatedAllocationsClient",
]



"""Types for the webhooks domain."""

from typing import Any, Literal, NotRequired, Optional, TypedDict, Union

class ListWebhooksResponse(TypedDict, total=False):
    """listWebhooks response."""

    items: list[TypedDict]
    next_page_token: NotRequired[str]

class ListWebhooksQuery(TypedDict, total=False):
    """listWebhooks query parameters."""

    limit: NotRequired[int]
    pagination_token: NotRequired[str]

class CreateWebhookRequest(TypedDict, total=False):
    """createWebhook request body."""

    url: str
    status: NotRequired[Literal["Enabled", "Disabled"]]
    description: NotRequired[str]
    events: list[Union[Literal["policy.triggered", "policy.approval.pending", "policy.approval.resolved", "key.created", "key.deleted", "key.delegated", "key.exported", "wallet.blockchainevent.detected", "wallet.created", "wallet.activated", "wallet.delegated", "wallet.exported", "wallet.signature.failed", "wallet.signature.rejected", "wallet.signature.requested", "wallet.signature.signed", "wallet.transaction.broadcasted", "wallet.transaction.confirmed", "wallet.transaction.failed", "wallet.transaction.rejected", "wallet.transaction.requested", "wallet.transfer.broadcasted", "wallet.transfer.confirmed", "wallet.transfer.failed", "wallet.transfer.rejected", "wallet.transfer.requested", "wallet.offer.received", "wallet.offer.accepted", "wallet.offer.rejected", "wallet.tags.modified"], Literal["*"]]]

class CreateWebhookResponse(TypedDict, total=False):
    """createWebhook response."""

    id: str
    url: str
    events: list[Union[Literal["policy.triggered", "policy.approval.pending", "policy.approval.resolved", "key.created", "key.deleted", "key.delegated", "key.exported", "wallet.blockchainevent.detected", "wallet.created", "wallet.activated", "wallet.delegated", "wallet.exported", "wallet.signature.failed", "wallet.signature.rejected", "wallet.signature.requested", "wallet.signature.signed", "wallet.transaction.broadcasted", "wallet.transaction.confirmed", "wallet.transaction.failed", "wallet.transaction.rejected", "wallet.transaction.requested", "wallet.transfer.broadcasted", "wallet.transfer.confirmed", "wallet.transfer.failed", "wallet.transfer.rejected", "wallet.transfer.requested", "wallet.offer.received", "wallet.offer.accepted", "wallet.offer.rejected", "wallet.tags.modified"], Literal["*"]]]
    status: Literal["Enabled", "Disabled"]
    description: NotRequired[str]
    date_created: str
    date_updated: str
    secret: str

class GetWebhookResponse(TypedDict, total=False):
    """getWebhook response."""

    id: str
    url: str
    events: list[Union[Literal["policy.triggered", "policy.approval.pending", "policy.approval.resolved", "key.created", "key.deleted", "key.delegated", "key.exported", "wallet.blockchainevent.detected", "wallet.created", "wallet.activated", "wallet.delegated", "wallet.exported", "wallet.signature.failed", "wallet.signature.rejected", "wallet.signature.requested", "wallet.signature.signed", "wallet.transaction.broadcasted", "wallet.transaction.confirmed", "wallet.transaction.failed", "wallet.transaction.rejected", "wallet.transaction.requested", "wallet.transfer.broadcasted", "wallet.transfer.confirmed", "wallet.transfer.failed", "wallet.transfer.rejected", "wallet.transfer.requested", "wallet.offer.received", "wallet.offer.accepted", "wallet.offer.rejected", "wallet.tags.modified"], Literal["*"]]]
    status: Literal["Enabled", "Disabled"]
    description: NotRequired[str]
    date_created: str
    date_updated: str

class UpdateWebhookRequest(TypedDict, total=False):
    """updateWebhook request body."""

    url: NotRequired[str]
    description: NotRequired[str]
    events: NotRequired[list[Union[Literal["policy.triggered", "policy.approval.pending", "policy.approval.resolved", "key.created", "key.deleted", "key.delegated", "key.exported", "wallet.blockchainevent.detected", "wallet.created", "wallet.activated", "wallet.delegated", "wallet.exported", "wallet.signature.failed", "wallet.signature.rejected", "wallet.signature.requested", "wallet.signature.signed", "wallet.transaction.broadcasted", "wallet.transaction.confirmed", "wallet.transaction.failed", "wallet.transaction.rejected", "wallet.transaction.requested", "wallet.transfer.broadcasted", "wallet.transfer.confirmed", "wallet.transfer.failed", "wallet.transfer.rejected", "wallet.transfer.requested", "wallet.offer.received", "wallet.offer.accepted", "wallet.offer.rejected", "wallet.tags.modified"], Literal["*"]]]]
    status: NotRequired[Literal["Enabled", "Disabled"]]

class UpdateWebhookResponse(TypedDict, total=False):
    """updateWebhook response."""

    id: str
    url: str
    events: list[Union[Literal["policy.triggered", "policy.approval.pending", "policy.approval.resolved", "key.created", "key.deleted", "key.delegated", "key.exported", "wallet.blockchainevent.detected", "wallet.created", "wallet.activated", "wallet.delegated", "wallet.exported", "wallet.signature.failed", "wallet.signature.rejected", "wallet.signature.requested", "wallet.signature.signed", "wallet.transaction.broadcasted", "wallet.transaction.confirmed", "wallet.transaction.failed", "wallet.transaction.rejected", "wallet.transaction.requested", "wallet.transfer.broadcasted", "wallet.transfer.confirmed", "wallet.transfer.failed", "wallet.transfer.rejected", "wallet.transfer.requested", "wallet.offer.received", "wallet.offer.accepted", "wallet.offer.rejected", "wallet.tags.modified"], Literal["*"]]]
    status: Literal["Enabled", "Disabled"]
    description: NotRequired[str]
    date_created: str
    date_updated: str

class DeleteWebhookResponse(TypedDict, total=False):
    """deleteWebhook response."""

    deleted: Literal[True]

class PingWebhookResponse(TypedDict, total=False):
    """pingWebhook response."""

    status: str
    error: NotRequired[str]

class GetWebhookEventResponse(TypedDict, total=False):
    """getWebhookEvent response."""

    id: str
    date: str
    kind: Literal["policy.triggered", "policy.approval.pending", "policy.approval.resolved", "key.created", "key.deleted", "key.delegated", "key.exported", "wallet.blockchainevent.detected", "wallet.created", "wallet.activated", "wallet.delegated", "wallet.exported", "wallet.signature.failed", "wallet.signature.rejected", "wallet.signature.requested", "wallet.signature.signed", "wallet.transaction.broadcasted", "wallet.transaction.confirmed", "wallet.transaction.failed", "wallet.transaction.rejected", "wallet.transaction.requested", "wallet.transfer.broadcasted", "wallet.transfer.confirmed", "wallet.transfer.failed", "wallet.transfer.rejected", "wallet.transfer.requested", "wallet.offer.received", "wallet.offer.accepted", "wallet.offer.rejected", "wallet.tags.modified"]
    data: dict[str, dict[str, Any]]
    status: str
    error: NotRequired[str]
    timestamp_sent: int

class ListWebhookEventsResponse(TypedDict, total=False):
    """listWebhookEvents response."""

    items: list[TypedDict]
    next_page_token: NotRequired[str]

class ListWebhookEventsQuery(TypedDict, total=False):
    """listWebhookEvents query parameters."""

    kind: NotRequired[Literal["policy.triggered", "policy.approval.pending", "policy.approval.resolved", "key.created", "key.deleted", "key.delegated", "key.exported", "wallet.blockchainevent.detected", "wallet.created", "wallet.activated", "wallet.delegated", "wallet.exported", "wallet.signature.failed", "wallet.signature.rejected", "wallet.signature.requested", "wallet.signature.signed", "wallet.transaction.broadcasted", "wallet.transaction.confirmed", "wallet.transaction.failed", "wallet.transaction.rejected", "wallet.transaction.requested", "wallet.transfer.broadcasted", "wallet.transfer.confirmed", "wallet.transfer.failed", "wallet.transfer.rejected", "wallet.transfer.requested", "wallet.offer.received", "wallet.offer.accepted", "wallet.offer.rejected", "wallet.tags.modified"]]
    delivery_failed: NotRequired[Literal["true", "false"]]
    limit: NotRequired[int]
    pagination_token: NotRequired[str]



"""Types for the policies domain."""

from typing import Any, Literal, NotRequired, Optional, TypedDict, Union

class GetPolicyResponse(TypedDict, total=False):
    """getPolicy response."""

    pending_change_request: NotRequired[TypedDict]

class CreateApprovalDecisionRequest(TypedDict, total=False):
    """createApprovalDecision request body."""

    value: Literal["Approved", "Denied"]
    reason: NotRequired[str]

class CreateApprovalDecisionResponse(TypedDict, total=False):
    """createApprovalDecision response."""

    id: str
    initiator_id: str
    activity: TypedDict
    status: Literal["Pending", "Approved", "Denied", "Expired"]
    expiration_date: NotRequired[str]
    date_created: NotRequired[str]
    date_updated: str
    date_resolved: NotRequired[str]
    policy_evaluations: list[TypedDict]
    decisions: list[TypedDict]

class ListPoliciesResponse(TypedDict, total=False):
    """listPolicies response."""

    items: list[TypedDict]
    next_page_token: NotRequired[str]

class ListPoliciesQuery(TypedDict, total=False):
    """listPolicies query parameters."""

    limit: NotRequired[str]
    pagination_token: NotRequired[str]
    status: NotRequired[Literal["Active", "Archived"]]

class GetApprovalResponse(TypedDict, total=False):
    """getApproval response."""

    id: str
    initiator_id: str
    activity: TypedDict
    status: Literal["Pending", "Approved", "Denied", "Expired"]
    expiration_date: NotRequired[str]
    date_created: NotRequired[str]
    date_updated: str
    date_resolved: NotRequired[str]
    policy_evaluations: list[TypedDict]
    decisions: list[TypedDict]

class ListApprovalsResponse(TypedDict, total=False):
    """listApprovals response."""

    items: list[TypedDict]
    next_page_token: NotRequired[str]

class ListApprovalsQuery(TypedDict, total=False):
    """listApprovals query parameters."""

    limit: NotRequired[str]
    pagination_token: NotRequired[str]
    status: NotRequired[Literal["Pending", "Approved", "Denied", "Expired"]]
    initiator_id: NotRequired[str]
    approver_id: NotRequired[str]



"""Delegated client for the policies domain."""

import json
from typing import Any, Literal, Optional, TypedDict, Union

from ..._internal import HttpClient
from ...base_auth_api import (
    BaseAuthApi,
    SignUserActionChallengeRequest,
    UserActionChallengeResponse,
)
from . import types as T


class DelegatedPoliciesClient:
    """
    Delegated client for policies operations.

    This client separates user action signing into _init() and _complete() method pairs,
    allowing external systems to handle the signing process.
    """

    def __init__(self, http_client: HttpClient):
        self._http = http_client

    def get_policy(self, policy_id: str) -> T.GetPolicyResponse:
        """
        Get Policy.

        Retrieve information about a specific policy.

        Args:
        policy_id: Path parameter.

        Returns:
            T.GetPolicyResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/v2/policies/{policyId}",
            path_params={"policyId": policy_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def update_policy_init(self, policy_id: str, body: dict[str, Any]) -> UserActionChallengeResponse:
        """
        Initialize Update Policy.

        Creates a user action challenge for external signing.

        Args:
        policy_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/v2/policies/{policyId}"
        path = path.replace("{policyId}", str(policy_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="PUT",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def update_policy_complete(self, policy_id: str, body: dict[str, Any], signed_challenge: SignUserActionChallengeRequest) -> TypedDict:
        """
        Complete Update Policy.

        Submits the signed challenge and makes the API request.

        Args:
        policy_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            TypedDict: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="PUT",
            path="/v2/policies/{policyId}",
            path_params={"policyId": policy_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def delete_policy_init(self, policy_id: str) -> UserActionChallengeResponse:
        """
        Initialize Delete Policy.

        Creates a user action challenge for external signing.

        Args:
        policy_id: Path parameter.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/v2/policies/{policyId}"
        path = path.replace("{policyId}", str(policy_id))
        payload = ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="DELETE",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def delete_policy_complete(self, policy_id: str, signed_challenge: SignUserActionChallengeRequest) -> TypedDict:
        """
        Complete Delete Policy.

        Submits the signed challenge and makes the API request.

        Args:
        policy_id: Path parameter.
        signed_challenge: The signed challenge from external signing.

        Returns:
            TypedDict: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="DELETE",
            path="/v2/policies/{policyId}",
            path_params={"policyId": policy_id},
            query_params=None,
            body=None,
            user_action=user_action_token,
        )

    def create_approval_decision_init(self, approval_id: str, body: T.CreateApprovalDecisionRequest) -> UserActionChallengeResponse:
        """
        Initialize Create Approval Decision.

        Creates a user action challenge for external signing.

        Args:
        approval_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/v2/policy-approvals/{approvalId}/decisions"
        path = path.replace("{approvalId}", str(approval_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def create_approval_decision_complete(self, approval_id: str, body: T.CreateApprovalDecisionRequest, signed_challenge: SignUserActionChallengeRequest) -> T.CreateApprovalDecisionResponse:
        """
        Complete Create Approval Decision.

        Submits the signed challenge and makes the API request.

        Args:
        approval_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.CreateApprovalDecisionResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/v2/policy-approvals/{approvalId}/decisions",
            path_params={"approvalId": approval_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def list_policies(self, query: Optional[T.ListPoliciesQuery] = None) -> T.ListPoliciesResponse:
        """
        List Policies.

        Retrieve the list of policies on your organization.

        Args:
        query: Query parameters.

        Returns:
            T.ListPoliciesResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/v2/policies",
            path_params={},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def create_policy_init(self, body: dict[str, Any]) -> UserActionChallengeResponse:
        """
        Initialize Create Policy.

        Creates a user action challenge for external signing.

        Args:
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/v2/policies"
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def create_policy_complete(self, body: dict[str, Any], signed_challenge: SignUserActionChallengeRequest) -> TypedDict:
        """
        Complete Create Policy.

        Submits the signed challenge and makes the API request.

        Args:
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            TypedDict: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/v2/policies",
            path_params={},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def get_approval(self, approval_id: str) -> T.GetApprovalResponse:
        """
        Get Approval.

        Retrieve information about a specific approval request.

        Args:
        approval_id: Path parameter.

        Returns:
            T.GetApprovalResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/v2/policy-approvals/{approvalId}",
            path_params={"approvalId": approval_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def list_approvals(self, query: Optional[T.ListApprovalsQuery] = None) -> T.ListApprovalsResponse:
        """
        List Approvals.

        Retrieve the list of pending approval requests.

        Args:
        query: Query parameters.

        Returns:
            T.ListApprovalsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/v2/policy-approvals",
            path_params={},
            query_params=query,
            body=None,
            requires_signature=False,
        )



"""Agreements domain module."""

from .client import AgreementsClient
from .delegated_client import DelegatedAgreementsClient
from . import types

__all__ = ["AgreementsClient", "DelegatedAgreementsClient", "types"]



"""Exchanges domain module."""

from .client import ExchangesClient
from .delegated_client import DelegatedExchangesClient
from . import types

__all__ = ["ExchangesClient", "DelegatedExchangesClient", "types"]



"""Delegated client for the permissions domain."""

import json
from typing import Any, Literal, Optional, TypedDict, Union

from ..._internal import HttpClient
from ...base_auth_api import (
    BaseAuthApi,
    SignUserActionChallengeRequest,
    UserActionChallengeResponse,
)
from . import types as T


class DelegatedPermissionsClient:
    """
    Delegated client for permissions operations.

    This client separates user action signing into _init() and _complete() method pairs,
    allowing external systems to handle the signing process.
    """

    def __init__(self, http_client: HttpClient):
        self._http = http_client

    def list_permission_assignments(self, permission_id: str) -> T.ListPermissionAssignmentsResponse:
        """
        List Permission Assignments.

        Retrieves a list of permission assignments (success) or gives a reason why it's not possible (failure).

        Args:
        permission_id: Path parameter.

        Returns:
            T.ListPermissionAssignmentsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/permissions/{permissionId}/assignments",
            path_params={"permissionId": permission_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def assign_permission_init(self, permission_id: str, body: T.AssignPermissionRequest) -> UserActionChallengeResponse:
        """
        Initialize Assign Permission.

        Creates a user action challenge for external signing.

        Args:
        permission_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/permissions/{permissionId}/assignments"
        path = path.replace("{permissionId}", str(permission_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def assign_permission_complete(self, permission_id: str, body: T.AssignPermissionRequest, signed_challenge: SignUserActionChallengeRequest) -> T.AssignPermissionResponse:
        """
        Complete Assign Permission.

        Submits the signed challenge and makes the API request.

        Args:
        permission_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.AssignPermissionResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/permissions/{permissionId}/assignments",
            path_params={"permissionId": permission_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def revoke_permission_init(self, permission_id: str, assignment_id: str, query: Optional[T.RevokePermissionQuery] = None) -> UserActionChallengeResponse:
        """
        Initialize Revoke Permission.

        Creates a user action challenge for external signing.

        Args:
        permission_id: Path parameter.
        assignment_id: Path parameter.
        query: Query parameters.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/permissions/{permissionId}/assignments/{assignmentId}"
        path = path.replace("{permissionId}", str(permission_id))
        path = path.replace("{assignmentId}", str(assignment_id))
        payload = ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="DELETE",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def revoke_permission_complete(self, permission_id: str, assignment_id: str, signed_challenge: SignUserActionChallengeRequest, query: Optional[T.RevokePermissionQuery] = None) -> None:
        """
        Complete Revoke Permission.

        Submits the signed challenge and makes the API request.

        Args:
        permission_id: Path parameter.
        assignment_id: Path parameter.
        signed_challenge: The signed challenge from external signing.
        query: Query parameters.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="DELETE",
            path="/permissions/{permissionId}/assignments/{assignmentId}",
            path_params={"permissionId": permission_id, "assignmentId": assignment_id},
            query_params=query,
            body=None,
            user_action=user_action_token,
        )

    def delete_permission_init(self, permission_id: str, body: T.DeletePermissionRequest) -> UserActionChallengeResponse:
        """
        Initialize Delete Permission.

        Creates a user action challenge for external signing.

        Args:
        permission_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/permissions/{permissionId}/archive"
        path = path.replace("{permissionId}", str(permission_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="PUT",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def delete_permission_complete(self, permission_id: str, body: T.DeletePermissionRequest, signed_challenge: SignUserActionChallengeRequest) -> T.DeletePermissionResponse:
        """
        Complete Delete Permission.

        Submits the signed challenge and makes the API request.

        Args:
        permission_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.DeletePermissionResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="PUT",
            path="/permissions/{permissionId}/archive",
            path_params={"permissionId": permission_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def list_permissions(self, query: Optional[T.ListPermissionsQuery] = None) -> T.ListPermissionsResponse:
        """
        List Permissions.

        Retrieves a list of permissions (success) or gives a reason why it's not possible (failure).

        Args:
        query: Query parameters.

        Returns:
            T.ListPermissionsResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/permissions",
            path_params={},
            query_params=query,
            body=None,
            requires_signature=False,
        )

    def create_permission_init(self, body: T.CreatePermissionRequest) -> UserActionChallengeResponse:
        """
        Initialize Create Permission.

        Creates a user action challenge for external signing.

        Args:
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/permissions"
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="POST",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def create_permission_complete(self, body: T.CreatePermissionRequest, signed_challenge: SignUserActionChallengeRequest) -> T.CreatePermissionResponse:
        """
        Complete Create Permission.

        Submits the signed challenge and makes the API request.

        Args:
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.CreatePermissionResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="POST",
            path="/permissions",
            path_params={},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )

    def get_permission(self, permission_id: str) -> T.GetPermissionResponse:
        """
        Get Permission.

        Retrieves a specific permission (success) or gives a reason why it's not possible (failure).

        Args:
        permission_id: Path parameter.

        Returns:
            T.GetPermissionResponse: The API response.
        """
        return self._http.request(
            method="GET",
            path="/permissions/{permissionId}",
            path_params={"permissionId": permission_id},
            query_params=None,
            body=None,
            requires_signature=False,
        )

    def update_permission_init(self, permission_id: str, body: T.UpdatePermissionRequest) -> UserActionChallengeResponse:
        """
        Initialize Update Permission.

        Creates a user action challenge for external signing.

        Args:
        permission_id: Path parameter.
        body: Request body.

        Returns:
            UserActionChallengeResponse: The challenge to sign externally.
        """
        path = "/permissions/{permissionId}"
        path = path.replace("{permissionId}", str(permission_id))
        payload = json.dumps(body, separators=(",", ":")) if body else ""

        return BaseAuthApi.create_user_action_challenge(
            self._http,
            user_action_http_method="PUT",
            user_action_http_path=path,
            user_action_payload=payload,
        )

    def update_permission_complete(self, permission_id: str, body: T.UpdatePermissionRequest, signed_challenge: SignUserActionChallengeRequest) -> T.UpdatePermissionResponse:
        """
        Complete Update Permission.

        Submits the signed challenge and makes the API request.

        Args:
        permission_id: Path parameter.
        body: Request body.
        signed_challenge: The signed challenge from external signing.

        Returns:
            T.UpdatePermissionResponse: The API response.
        """
        user_action_result = BaseAuthApi.sign_user_action_challenge(
            self._http, signed_challenge
        )
        user_action_token = user_action_result["userAction"]

        return self._http.request_with_user_action(
            method="PUT",
            path="/permissions/{permissionId}",
            path_params={"permissionId": permission_id},
            query_params=None,
            body=body,
            user_action=user_action_token,
        )
